/*******************************************************************************
* File Name: Temp_Sensor.c
* Version 3.30
*
* Description:
*  This file provides the source code of APIs for the I2C component.
*  Actual protocol and operation code resides in the interrupt service routine
*  file.
*
* Note:
*
*******************************************************************************
* Copyright 2008-2012, Cypress Semiconductor Corporation. All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "Temp_Sensor_PVT.h"


/**********************************
*      System variables
**********************************/

uint8 Temp_Sensor_initVar = 0u;    /* Defines if component was initialized */

volatile uint8 Temp_Sensor_state;  /* Current state of I2C FSM */


/*******************************************************************************
* Function Name: Temp_Sensor_Init
********************************************************************************
*
* Summary:
*  Initializes I2C registers with initial values provided from customizer.
*
* Parameters:
*  None
*
* Return:
*  None
*
* Global variables:
*  None
*
* Reentrant:
*  No
*
*******************************************************************************/
void Temp_Sensor_Init(void) 
{
    #if(Temp_Sensor_FF_IMPLEMENTED)
        Temp_Sensor_CFG_REG  = Temp_Sensor_DEFAULT_CFG;
        Temp_Sensor_XCFG_REG = Temp_Sensor_DEFAULT_XCFG;

        #if(CY_PSOC5A)
            Temp_Sensor_CLKDIV_REG  = LO8(Temp_Sensor_DEFAULT_DIVIDE_FACTOR);
        #else
            Temp_Sensor_CLKDIV1_REG = LO8(Temp_Sensor_DEFAULT_DIVIDE_FACTOR);
            Temp_Sensor_CLKDIV2_REG = HI8(Temp_Sensor_DEFAULT_DIVIDE_FACTOR);
        #endif /* (CY_PSOC5A) */

    #else
        uint8 enableInterrupts;

        Temp_Sensor_CFG_REG      = Temp_Sensor_DEFAULT_CFG;      /* control  */
        Temp_Sensor_INT_MASK_REG = Temp_Sensor_DEFAULT_INT_MASK; /* int_mask */

        /* Enable interrupts from block */
        enableInterrupts = CyEnterCriticalSection();
        Temp_Sensor_INT_ENABLE_REG |= Temp_Sensor_INTR_ENABLE; /* aux_ctl */
        CyExitCriticalSection(enableInterrupts);

        #if(Temp_Sensor_MODE_MASTER_ENABLED)
            Temp_Sensor_MCLK_PRD_REG = Temp_Sensor_DEFAULT_MCLK_PRD;
            Temp_Sensor_MCLK_CMP_REG = Temp_Sensor_DEFAULT_MCLK_CMP;
         #endif /* (Temp_Sensor_MODE_MASTER_ENABLED) */

        #if(Temp_Sensor_MODE_SLAVE_ENABLED)
            Temp_Sensor_PERIOD_REG = Temp_Sensor_DEFAULT_PERIOD;
        #endif  /* (Temp_Sensor_MODE_SLAVE_ENABLED) */

    #endif /* (Temp_Sensor_FF_IMPLEMENTED) */

    #if(Temp_Sensor_TIMEOUT_ENABLED)
        Temp_Sensor_TimeoutInit();
    #endif /* (Temp_Sensor_TIMEOUT_ENABLED) */

    /* Disable Interrupt and set vector and priority */
    CyIntDisable    (Temp_Sensor_ISR_NUMBER);
    CyIntSetPriority(Temp_Sensor_ISR_NUMBER, Temp_Sensor_ISR_PRIORITY);
    #if(Temp_Sensor_INTERN_I2C_INTR_HANDLER)
        (void) CyIntSetVector(Temp_Sensor_ISR_NUMBER, &Temp_Sensor_ISR);
    #endif /* (Temp_Sensor_INTERN_I2C_INTR_HANDLER) */


    /* Put state machine in idle state */
    Temp_Sensor_state = Temp_Sensor_SM_IDLE;

    #if(Temp_Sensor_MODE_SLAVE_ENABLED)
        /* Reset status and buffers index */
        Temp_Sensor_SlaveClearReadBuf();
        Temp_Sensor_SlaveClearWriteBuf();
        Temp_Sensor_slStatus = 0u; /* Reset slave status */

        /* Set default address */
        Temp_Sensor_SlaveSetAddress(Temp_Sensor_DEFAULT_ADDR);
    #endif /* (Temp_Sensor_MODE_SLAVE_ENABLED) */

    #if(Temp_Sensor_MODE_MASTER_ENABLED)
        /* Reset status and buffers index */
        Temp_Sensor_MasterClearReadBuf();
        Temp_Sensor_MasterClearWriteBuf();
        (void) Temp_Sensor_MasterClearStatus();
    #endif /* (Temp_Sensor_MODE_MASTER_ENABLED) */
}


/*******************************************************************************
* Function Name: Temp_Sensor_Enable
********************************************************************************
*
* Summary:
*  Enables I2C operations.
*
* Parameters:
*  None
*
* Return:
*  None
*
* Global variables:
*  None
*
*******************************************************************************/
void Temp_Sensor_Enable(void) 
{
    #if(Temp_Sensor_FF_IMPLEMENTED)
        uint8 enableInterrupts;

        /* Enable power to I2C FF block */
        enableInterrupts = CyEnterCriticalSection();
        Temp_Sensor_ACT_PWRMGR_REG  |= Temp_Sensor_ACT_PWR_EN;
        Temp_Sensor_STBY_PWRMGR_REG |= Temp_Sensor_STBY_PWR_EN;
        CyExitCriticalSection(enableInterrupts);

    #else

        #if(Temp_Sensor_MODE_SLAVE_ENABLED)
            uint8 enableInterrupts;
        #endif /* (Temp_Sensor_MODE_SLAVE_ENABLED) */

        #if(Temp_Sensor_MODE_SLAVE_ENABLED)
            /* Enable slave bit counter */
            enableInterrupts = CyEnterCriticalSection();
            Temp_Sensor_COUNTER_AUX_CTL_REG |= Temp_Sensor_CNT7_ENABLE;   /* aux_ctl */
            CyExitCriticalSection(enableInterrupts);
        #endif /* (Temp_Sensor_MODE_SLAVE_ENABLED) */

        Temp_Sensor_CFG_REG |= Temp_Sensor_ENABLE_MS;

    #endif /* (Temp_Sensor_FF_IMPLEMENTED) */

    #if(Temp_Sensor_TIMEOUT_ENABLED)
        Temp_Sensor_TimeoutEnable();
    #endif /* (Temp_Sensor_TIMEOUT_ENABLED) */
}


/*******************************************************************************
* Function Name: Temp_Sensor_Start
********************************************************************************
*
* Summary:
*  Starts the I2C hardware. Enables Active mode power template bits or clock
*  gating as appropriate. It is required to be executed before I2C bus
*  operation.
*  The I2C interrupt remains disabled after this function call.
*
* Parameters:
*  None
*
* Return:
*  None
*
* Side Effects:
*  This component automatically enables it's interrupt.  If I2C is enabled
*  without the interrupt enabled, it could lock up the I2C bus.
*
* Global variables:
*  Temp_Sensor_initVar - used to check initial configuration, modified
*  on first function call.
*
* Reentrant:
*  No
*
*******************************************************************************/
void Temp_Sensor_Start(void) 
{
    /* Initialize I2C registers, reset I2C buffer index and clears status */
    if(0u == Temp_Sensor_initVar)
    {
        Temp_Sensor_Init();
        Temp_Sensor_initVar = 1u; /* Component initialized */
    }

    Temp_Sensor_Enable();
    Temp_Sensor_EnableInt();
}


/*******************************************************************************
* Function Name: Temp_Sensor_Stop
********************************************************************************
*
* Summary:
*  Disables I2C hardware and disables I2C interrupt. Disables Active mode power
*  template bits or clock gating as appropriate.
*
* Parameters:
*  None
*
* Return:
*  None
*
*******************************************************************************/
void Temp_Sensor_Stop(void) 
{
    #if((Temp_Sensor_FF_IMPLEMENTED)  || \
        (Temp_Sensor_UDB_IMPLEMENTED && Temp_Sensor_MODE_SLAVE_ENABLED))
        uint8 enableInterrupts;
    #endif /* ((Temp_Sensor_FF_IMPLEMENTED)  || \
               (Temp_Sensor_UDB_IMPLEMENTED && Temp_Sensor_MODE_SLAVE_ENABLED)) */

    Temp_Sensor_DisableInt();

    Temp_Sensor_DISABLE_INT_ON_STOP;   /* Interrupt on Stop can be enabled by write */
    (void) Temp_Sensor_CSR_REG;        /* Clear CSR reg */
    
    #if(Temp_Sensor_TIMEOUT_ENABLED)
        Temp_Sensor_TimeoutStop();
    #endif  /* End (Temp_Sensor_TIMEOUT_ENABLED) */

    #if(Temp_Sensor_FF_IMPLEMENTED)
        #if(CY_PSOC3 || CY_PSOC5LP)
            /* Store registers which are held in reset when Master and Slave bits are cleared */
            #if(Temp_Sensor_MODE_SLAVE_ENABLED)
                Temp_Sensor_backup.addr = Temp_Sensor_ADDR_REG;
            #endif /* (Temp_Sensor_MODE_SLAVE_ENABLED) */

            Temp_Sensor_backup.clkDiv1  = Temp_Sensor_CLKDIV1_REG;
            Temp_Sensor_backup.clkDiv2  = Temp_Sensor_CLKDIV2_REG;


            /* Reset FF block */
            Temp_Sensor_CFG_REG &= ((uint8) ~Temp_Sensor_ENABLE_MS);
            CyDelayUs(Temp_Sensor_FF_RESET_DELAY);
            Temp_Sensor_CFG_REG |= ((uint8)  Temp_Sensor_ENABLE_MS);


            /* Restore registers */
            #if(Temp_Sensor_MODE_SLAVE_ENABLED)
                Temp_Sensor_ADDR_REG = Temp_Sensor_backup.addr;
            #endif /* (Temp_Sensor_MODE_SLAVE_ENABLED) */

            Temp_Sensor_CLKDIV1_REG = Temp_Sensor_backup.clkDiv1;
            Temp_Sensor_CLKDIV2_REG = Temp_Sensor_backup.clkDiv2;

        #endif /* (CY_PSOC3 || CY_PSOC5LP) */

        /* Disable power to I2C block */
        enableInterrupts = CyEnterCriticalSection();
        Temp_Sensor_ACT_PWRMGR_REG  &= ((uint8) ~Temp_Sensor_ACT_PWR_EN);
        Temp_Sensor_STBY_PWRMGR_REG &= ((uint8) ~Temp_Sensor_STBY_PWR_EN);
        CyExitCriticalSection(enableInterrupts);

    #else

        #if(Temp_Sensor_MODE_SLAVE_ENABLED)
            /* Disable slave bit counter */
            enableInterrupts = CyEnterCriticalSection();
            Temp_Sensor_COUNTER_AUX_CTL_REG &= ((uint8) ~Temp_Sensor_CNT7_ENABLE);
            CyExitCriticalSection(enableInterrupts);
        #endif /* (Temp_Sensor_MODE_SLAVE_ENABLED) */

        Temp_Sensor_CFG_REG &= ((uint8) ~Temp_Sensor_ENABLE_MS);

    #endif /* (Temp_Sensor_FF_IMPLEMENTED) */

    Temp_Sensor_ClearPendingInt();  /* Clear interrupt triggers on reset */

    Temp_Sensor_state = Temp_Sensor_SM_IDLE;  /* Reset software FSM */
}


/* [] END OF FILE */
