/*******************************************************************************
* File Name: Temp_Sensor_INT.c
* Version 3.30
*
* Description:
*  This file provides the source code of Interrupt Service Routine (ISR)
*  for I2C component.
*
*  Note:
*
********************************************************************************
* Copyright 2008-2012, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "Temp_Sensor_PVT.h"


/*******************************************************************************
*  Place your includes, defines and code here
********************************************************************************/
/* `#START Temp_Sensor_ISR_intc` */

/* `#END` */


/*******************************************************************************
* Function Name: Temp_Sensor_ISR
********************************************************************************
*
* Summary:
*  Handler for I2C interrupt. The Slave and Master operations are handled here.
*
* Parameters:
*  void
*
* Return:
*  void
*
* Reentrant:
*  No
*
*******************************************************************************/
CY_ISR(Temp_Sensor_ISR)
{
    #if(Temp_Sensor_MODE_SLAVE_ENABLED)
       uint8  tmp8;
    #endif  /* (Temp_Sensor_MODE_SLAVE_ENABLED) */

    uint8  tmpCsr;

    #if(Temp_Sensor_TIMEOUT_FF_ENABLED)
        if(0u != Temp_Sensor_TimeoutGetStatus())
        {
            Temp_Sensor_TimeoutReset();
            Temp_Sensor_state = Temp_Sensor_SM_EXIT_IDLE;
            /* Temp_Sensor_CSR_REG should be cleared after reset */
        }
    #endif /* (Temp_Sensor_TIMEOUT_FF_ENABLED) */


    tmpCsr = Temp_Sensor_CSR_REG;      /* Make copy as interrupts clear */

    #if(Temp_Sensor_MODE_MULTI_MASTER_SLAVE_ENABLED)
        if(Temp_Sensor_CHECK_START_GEN(Temp_Sensor_MCSR_REG))
        {
            Temp_Sensor_CLEAR_START_GEN;

            /* Set READ complete, but was aborted */
            Temp_Sensor_mstrStatus |= (Temp_Sensor_MSTAT_ERR_XFER |
                                            Temp_Sensor_GET_MSTAT_CMPLT);

            /* The slave was addressed */
            Temp_Sensor_state = Temp_Sensor_SM_SLAVE;
        }
    #endif /* (Temp_Sensor_MODE_MULTI_MASTER_SLAVE_ENABLED) */


    #if(Temp_Sensor_MODE_MULTI_MASTER_ENABLED)
        if(Temp_Sensor_CHECK_LOST_ARB(tmpCsr))
        {
            /* Set errors */
            Temp_Sensor_mstrStatus |= (Temp_Sensor_MSTAT_ERR_XFER     |
                                            Temp_Sensor_MSTAT_ERR_ARB_LOST |
                                            Temp_Sensor_GET_MSTAT_CMPLT);

            Temp_Sensor_DISABLE_INT_ON_STOP; /* Interrupt on Stop is enabled by write */

            #if(Temp_Sensor_MODE_MULTI_MASTER_SLAVE_ENABLED)
                if(Temp_Sensor_CHECK_ADDRESS_STS(tmpCsr))
                {
                    /* The slave was addressed */
                    Temp_Sensor_state = Temp_Sensor_SM_SLAVE;
                }
                else
                {
                    Temp_Sensor_BUS_RELEASE;

                    Temp_Sensor_state = Temp_Sensor_SM_EXIT_IDLE;
                }
            #else
                Temp_Sensor_BUS_RELEASE;

                Temp_Sensor_state = Temp_Sensor_SM_EXIT_IDLE;

            #endif /* (Temp_Sensor_MODE_MULTI_MASTER_SLAVE_ENABLED) */
        }
    #endif /* (Temp_Sensor_MODE_MULTI_MASTER_ENABLED) */

    /* Check for Master operation mode */
    if(Temp_Sensor_CHECK_SM_MASTER)
    {
        #if(Temp_Sensor_MODE_MASTER_ENABLED)
            if(Temp_Sensor_CHECK_BYTE_COMPLETE(tmpCsr))
            {
                switch (Temp_Sensor_state)
                {
                case Temp_Sensor_SM_MSTR_WR_ADDR:  /* After address is sent, WRITE data */
                case Temp_Sensor_SM_MSTR_RD_ADDR:  /* After address is sent, READ  data */

                    tmpCsr &= ((uint8) ~Temp_Sensor_CSR_STOP_STATUS); /* Clear STOP bit history on address phase */
                    
                    if(Temp_Sensor_CHECK_ADDR_ACK(tmpCsr))
                    {
                        /* Setup for transmit or receive of data */
                        if(Temp_Sensor_state == Temp_Sensor_SM_MSTR_WR_ADDR)   /* TRANSMIT data */
                        {
                            /* Check if at least one byte to transfer */
                            if(Temp_Sensor_mstrWrBufSize > 0u)
                            {
                                /* Load the 1st data byte */
                                Temp_Sensor_DATA_REG = Temp_Sensor_mstrWrBufPtr[0u];
                                Temp_Sensor_TRANSMIT_DATA;
                                Temp_Sensor_mstrWrBufIndex = 1u;   /* Set index to 2nd element */

                                /* Set transmit state until done */
                                Temp_Sensor_state = Temp_Sensor_SM_MSTR_WR_DATA;
                            }
                            /* End of buffer: complete writing */
                            else if(Temp_Sensor_CHECK_NO_STOP(Temp_Sensor_mstrControl))
                            {
                                #if(CY_PSOC5A)
                                    /* Do not handles 0 bytes transfer - HALT is NOT allowed */
                                    Temp_Sensor_ENABLE_INT_ON_STOP;
                                    Temp_Sensor_GENERATE_STOP;
                                
                                #else
                                    /* Set WRITE complete and Master HALTED */
                                    Temp_Sensor_mstrStatus |= (Temp_Sensor_MSTAT_XFER_HALT |
                                                                    Temp_Sensor_MSTAT_WR_CMPLT);

                                    Temp_Sensor_state = Temp_Sensor_SM_MSTR_HALT; /* Expect RESTART */
                                    Temp_Sensor_DisableInt();
                                
                                #endif /* (CY_PSOC5A) */
                            }
                            else
                            {
                                Temp_Sensor_ENABLE_INT_ON_STOP; /* Enable interrupt on STOP, to catch it */
                                Temp_Sensor_GENERATE_STOP;
                            }
                        }
                        else  /* Master Receive data */
                        {
                            Temp_Sensor_READY_TO_READ; /* Release bus to read data */

                            Temp_Sensor_state  = Temp_Sensor_SM_MSTR_RD_DATA;
                        }
                    }
                    /* Address is NACKed */
                    else if(Temp_Sensor_CHECK_ADDR_NAK(tmpCsr))
                    {
                        /* Set Address NAK error */
                        Temp_Sensor_mstrStatus |= (Temp_Sensor_MSTAT_ERR_XFER |
                                                        Temp_Sensor_MSTAT_ERR_ADDR_NAK);
                                                        
                        if(Temp_Sensor_CHECK_NO_STOP(Temp_Sensor_mstrControl))
                        {
                            Temp_Sensor_mstrStatus |= (Temp_Sensor_MSTAT_XFER_HALT | 
                                                            Temp_Sensor_GET_MSTAT_CMPLT);

                            Temp_Sensor_state = Temp_Sensor_SM_MSTR_HALT; /* Expect RESTART */
                            Temp_Sensor_DisableInt();
                        }
                        else  /* Do normal Stop */
                        {
                            Temp_Sensor_ENABLE_INT_ON_STOP; /* Enable interrupt on STOP, to catch it */
                            Temp_Sensor_GENERATE_STOP;
                        }
                    }
                    else
                    {
                        /* Address phase is not set for some reason: error */
                        #if(Temp_Sensor_TIMEOUT_ENABLED)
                            /* Exit from interrupt to take a chance for timeout timer handle this case */
                            Temp_Sensor_DisableInt();
                            Temp_Sensor_ClearPendingInt();
                        #else
                            /* Block execution flow: unexpected condition */
                            CYASSERT(0u != 0u);
                        #endif /* (Temp_Sensor_TIMEOUT_ENABLED) */
                    }
                    break;

                case Temp_Sensor_SM_MSTR_WR_DATA:

                    if(Temp_Sensor_CHECK_DATA_ACK(tmpCsr))
                    {
                        /* Check if end of buffer */
                        if(Temp_Sensor_mstrWrBufIndex  < Temp_Sensor_mstrWrBufSize)
                        {
                            Temp_Sensor_DATA_REG =
                                                     Temp_Sensor_mstrWrBufPtr[Temp_Sensor_mstrWrBufIndex];
                            Temp_Sensor_TRANSMIT_DATA;
                            Temp_Sensor_mstrWrBufIndex++;
                        }
                        /* End of buffer: complete writing */
                        else if(Temp_Sensor_CHECK_NO_STOP(Temp_Sensor_mstrControl))
                        {
                            /* Set WRITE complete and Master HALTED */
                            Temp_Sensor_mstrStatus |= (Temp_Sensor_MSTAT_XFER_HALT |
                                                            Temp_Sensor_MSTAT_WR_CMPLT);

                            Temp_Sensor_state = Temp_Sensor_SM_MSTR_HALT;    /* Expect RESTART */
                            Temp_Sensor_DisableInt();
                        }
                        else  /* Do normal STOP */
                        {
                            Temp_Sensor_Workaround();          /* Workaround: empty function */
                            Temp_Sensor_ENABLE_INT_ON_STOP;    /* Enable interrupt on STOP, to catch it */
                            Temp_Sensor_GENERATE_STOP;
                        }
                    }
                    /* Last byte NAKed: end writing */
                    else if(Temp_Sensor_CHECK_NO_STOP(Temp_Sensor_mstrControl))
                    {
                        /* Set WRITE complete, SHORT transfer and Master HALTED */
                        Temp_Sensor_mstrStatus |= (Temp_Sensor_MSTAT_ERR_XFER       |
                                                        Temp_Sensor_MSTAT_ERR_SHORT_XFER |
                                                        Temp_Sensor_MSTAT_XFER_HALT      |
                                                        Temp_Sensor_MSTAT_WR_CMPLT);

                        Temp_Sensor_state = Temp_Sensor_SM_MSTR_HALT;    /* Expect RESTART */
                        Temp_Sensor_DisableInt();
                    }
                    else  /* Do normal STOP */
                    {
                        Temp_Sensor_ENABLE_INT_ON_STOP;    /* Enable interrupt on STOP, to catch it */
                        Temp_Sensor_GENERATE_STOP;

                        /* Set SHORT and ERR transfer */
                        Temp_Sensor_mstrStatus |= (Temp_Sensor_MSTAT_ERR_SHORT_XFER |
                                                        Temp_Sensor_MSTAT_ERR_XFER);
                    }
                    
                    break;

                case Temp_Sensor_SM_MSTR_RD_DATA:

                    Temp_Sensor_mstrRdBufPtr[Temp_Sensor_mstrRdBufIndex] = Temp_Sensor_DATA_REG;
                    Temp_Sensor_mstrRdBufIndex++;

                    /* Check if end of buffer */
                    if(Temp_Sensor_mstrRdBufIndex < Temp_Sensor_mstrRdBufSize)
                    {
                        Temp_Sensor_ACK_AND_RECEIVE;       /* ACK and receive byte */
                    }
                    /* End of buffer: complete reading */
                    else if(Temp_Sensor_CHECK_NO_STOP(Temp_Sensor_mstrControl))
                    {                        
                        /* Set READ complete and Master HALTED */
                        Temp_Sensor_mstrStatus |= (Temp_Sensor_MSTAT_XFER_HALT |
                                                        Temp_Sensor_MSTAT_RD_CMPLT);
                        
                        Temp_Sensor_state = Temp_Sensor_SM_MSTR_HALT;    /* Expect RESTART */
                        Temp_Sensor_DisableInt();
                    }
                    else
                    {
                        Temp_Sensor_ENABLE_INT_ON_STOP;
                        Temp_Sensor_NAK_AND_RECEIVE;       /* NACK and TRY to generate STOP */
                    }
                    break;

                default: /* This is an invalid state and should not occur */

                    #if(Temp_Sensor_TIMEOUT_ENABLED)
                        /* Exit from interrupt to take a chance for timeout timer handle this case */
                        Temp_Sensor_DisableInt();
                        Temp_Sensor_ClearPendingInt();
                    #else
                        /* Block execution flow: unexpected condition */
                        CYASSERT(0u != 0u);
                    #endif /* (Temp_Sensor_TIMEOUT_ENABLED) */

                    break;
                }
            }

            /* Catches the Stop: end of transaction */
            if(Temp_Sensor_CHECK_STOP_STS(tmpCsr))
            {
                Temp_Sensor_mstrStatus |= Temp_Sensor_GET_MSTAT_CMPLT;

                Temp_Sensor_DISABLE_INT_ON_STOP;
                Temp_Sensor_state = Temp_Sensor_SM_IDLE;
            }
        #endif /* (Temp_Sensor_MODE_MASTER_ENABLED) */
    }
    else if(Temp_Sensor_CHECK_SM_SLAVE)
    {
        #if(Temp_Sensor_MODE_SLAVE_ENABLED)
            
            if((Temp_Sensor_CHECK_STOP_STS(tmpCsr)) || /* Stop || Restart */
               (Temp_Sensor_CHECK_BYTE_COMPLETE(tmpCsr) && Temp_Sensor_CHECK_ADDRESS_STS(tmpCsr)))
            {
                /* Catch end of master write transcation: use interrupt on Stop */
                /* The STOP bit history on address phase does not have correct state */
                if(Temp_Sensor_SM_SL_WR_DATA == Temp_Sensor_state)
                {
                    Temp_Sensor_DISABLE_INT_ON_STOP;

                    Temp_Sensor_slStatus &= ((uint8) ~Temp_Sensor_SSTAT_WR_BUSY);
                    Temp_Sensor_slStatus |= ((uint8)  Temp_Sensor_SSTAT_WR_CMPLT);

                    Temp_Sensor_state = Temp_Sensor_SM_IDLE;
                }
            }

            if(Temp_Sensor_CHECK_BYTE_COMPLETE(tmpCsr))
            {
                /* The address only issued after Start or ReStart: so check address
                   to catch this events:
                    FF : sets Addr phase with byte_complete interrupt trigger.
                    UDB: sets Addr phase immediately after Start or ReStart. */
                if(Temp_Sensor_CHECK_ADDRESS_STS(tmpCsr))
                {
                    /* Check for software address detection */
                    #if(Temp_Sensor_SW_ADRR_DECODE)
                        tmp8 = Temp_Sensor_GET_SLAVE_ADDR(Temp_Sensor_DATA_REG);

                        if(tmp8 == Temp_Sensor_slAddress)   /* Check for address match */
                        {
                            if(0u != (Temp_Sensor_DATA_REG & Temp_Sensor_READ_FLAG))
                            {
                                /* Place code to prepare read buffer here                  */
                                /* `#START Temp_Sensor_SW_PREPARE_READ_BUF_interrupt` */

                                /* `#END` */

                                /* Prepare next opeation to read, get data and place in data register */
                                if(Temp_Sensor_slRdBufIndex < Temp_Sensor_slRdBufSize)
                                {
                                    /* Load first data byte from array */
                                    Temp_Sensor_DATA_REG = Temp_Sensor_slRdBufPtr[Temp_Sensor_slRdBufIndex];
                                    Temp_Sensor_ACK_AND_TRANSMIT;
                                    Temp_Sensor_slRdBufIndex++;

                                    Temp_Sensor_slStatus |= Temp_Sensor_SSTAT_RD_BUSY;
                                }
                                else    /* Overflow: provide 0xFF on the bus */
                                {
                                    Temp_Sensor_DATA_REG = Temp_Sensor_OVERFLOW_RETURN;
                                    Temp_Sensor_ACK_AND_TRANSMIT;

                                    Temp_Sensor_slStatus  |= (Temp_Sensor_SSTAT_RD_BUSY |
                                                                   Temp_Sensor_SSTAT_RD_ERR_OVFL);
                                }

                                Temp_Sensor_state = Temp_Sensor_SM_SL_RD_DATA;
                            }
                            else  /* Write transaction: receive 1st byte */
                            {
                                Temp_Sensor_ACK_AND_RECEIVE;
                                Temp_Sensor_state = Temp_Sensor_SM_SL_WR_DATA;

                                Temp_Sensor_slStatus |= Temp_Sensor_SSTAT_WR_BUSY;
                                Temp_Sensor_ENABLE_INT_ON_STOP;
                            }
                        }    
                        else
                        {
                            /*     Place code to compare for additional address here    */
                            /* `#START Temp_Sensor_SW_ADDR_COMPARE_interruptStart` */

                            /* `#END` */
                            
                            Temp_Sensor_NAK_AND_RECEIVE;   /* NACK address */

                            /* Place code to end of condition for NACK generation here */
                            /* `#START Temp_Sensor_SW_ADDR_COMPARE_interruptEnd`  */

                            /* `#END` */
                        }
                        
                    #else /* (Temp_Sensor_HW_ADRR_DECODE) */
                        
                        if(0u != (Temp_Sensor_DATA_REG & Temp_Sensor_READ_FLAG))
                        {
                            /* Place code to prepare read buffer here                  */
                            /* `#START Temp_Sensor_HW_PREPARE_READ_BUF_interrupt` */

                            /* `#END` */

                            /* Prepare next opeation to read, get data and place in data register */
                            if(Temp_Sensor_slRdBufIndex < Temp_Sensor_slRdBufSize)
                            {
                                /* Load first data byte from array */
                                Temp_Sensor_DATA_REG = Temp_Sensor_slRdBufPtr[Temp_Sensor_slRdBufIndex];
                                Temp_Sensor_ACK_AND_TRANSMIT;
                                Temp_Sensor_slRdBufIndex++;

                                Temp_Sensor_slStatus |= Temp_Sensor_SSTAT_RD_BUSY;
                            }
                            else    /* Overflow: provide 0xFF on the bus */
                            {
                                Temp_Sensor_DATA_REG = Temp_Sensor_OVERFLOW_RETURN;
                                Temp_Sensor_ACK_AND_TRANSMIT;

                                Temp_Sensor_slStatus  |= (Temp_Sensor_SSTAT_RD_BUSY |
                                                               Temp_Sensor_SSTAT_RD_ERR_OVFL);
                            }

                            Temp_Sensor_state = Temp_Sensor_SM_SL_RD_DATA;
                        }
                        else  /* Write transaction: receive 1st byte */
                        {
                            Temp_Sensor_ACK_AND_RECEIVE;
                            Temp_Sensor_state = Temp_Sensor_SM_SL_WR_DATA;

                            Temp_Sensor_slStatus |= Temp_Sensor_SSTAT_WR_BUSY;
                            Temp_Sensor_ENABLE_INT_ON_STOP;
                        }
                        
                    #endif /* (Temp_Sensor_SW_ADRR_DECODE) */
                }
                /* Data states */
                /* Data master writes into slave */
                else if(Temp_Sensor_state == Temp_Sensor_SM_SL_WR_DATA)
                {
                    if(Temp_Sensor_slWrBufIndex < Temp_Sensor_slWrBufSize)
                    {
                        tmp8 = Temp_Sensor_DATA_REG;
                        Temp_Sensor_ACK_AND_RECEIVE;
                        Temp_Sensor_slWrBufPtr[Temp_Sensor_slWrBufIndex] = tmp8;
                        Temp_Sensor_slWrBufIndex++;
                    }
                    else  /* of array: complete write, send NACK */
                    {
                        Temp_Sensor_NAK_AND_RECEIVE;

                        Temp_Sensor_slStatus |= Temp_Sensor_SSTAT_WR_ERR_OVFL;
                    }
                }
                /* Data master reads from slave */
                else if(Temp_Sensor_state == Temp_Sensor_SM_SL_RD_DATA)
                {
                    if(Temp_Sensor_CHECK_DATA_ACK(tmpCsr))
                    {
                        if(Temp_Sensor_slRdBufIndex < Temp_Sensor_slRdBufSize)
                        {
                             /* Get data from array */
                            Temp_Sensor_DATA_REG = Temp_Sensor_slRdBufPtr[Temp_Sensor_slRdBufIndex];
                            Temp_Sensor_TRANSMIT_DATA;
                            Temp_Sensor_slRdBufIndex++;
                        }
                        else   /* Overflow: provide 0xFF on the bus */
                        {
                            Temp_Sensor_DATA_REG = Temp_Sensor_OVERFLOW_RETURN;
                            Temp_Sensor_TRANSMIT_DATA;

                            Temp_Sensor_slStatus |= Temp_Sensor_SSTAT_RD_ERR_OVFL;
                        }
                    }
                    else  /* Last byte was NACKed: read complete */
                    {
                        /* Only NACK appears on the bus */
                        Temp_Sensor_DATA_REG = Temp_Sensor_OVERFLOW_RETURN;
                        Temp_Sensor_NAK_AND_TRANSMIT;

                        Temp_Sensor_slStatus &= ((uint8) ~Temp_Sensor_SSTAT_RD_BUSY);
                        Temp_Sensor_slStatus |= ((uint8)  Temp_Sensor_SSTAT_RD_CMPLT);

                        Temp_Sensor_state = Temp_Sensor_SM_IDLE;
                    }
                }
                else
                {
                    #if(Temp_Sensor_TIMEOUT_ENABLED)
                        /* Exit from interrupt to take a chance for timeout timer handle this case */
                        Temp_Sensor_DisableInt();
                        Temp_Sensor_ClearPendingInt();
                    #else
                        /* Block execution flow: unexpected condition */
                        CYASSERT(0u != 0u);
                    #endif /* (Temp_Sensor_TIMEOUT_ENABLED) */
                }
            }
        #endif /* (Temp_Sensor_MODE_SLAVE_ENABLED) */
    }
    else
    {
        /* The FSM skips master and slave processing: return to IDLE */
        Temp_Sensor_state = Temp_Sensor_SM_IDLE;
    }
}


#if((Temp_Sensor_FF_IMPLEMENTED) && (Temp_Sensor_WAKEUP_ENABLED))
    /*******************************************************************************
    * Function Name: Temp_Sensor_WAKEUP_ISR
    ********************************************************************************
    *
    * Summary:
    *  Empty interrupt handler to trigger after wakeup.
    *
    * Parameters:
    *  void
    *
    * Return:
    *  void
    *
    *******************************************************************************/
    CY_ISR(Temp_Sensor_WAKEUP_ISR)
    {
        Temp_Sensor_wakeupSource = 1u;  /* I2C was wakeup source */
        /* The SCL is stretched unitl the I2C_Wake() is called */
    }
#endif /* ((Temp_Sensor_FF_IMPLEMENTED) && (Temp_Sensor_WAKEUP_ENABLED))*/


/* [] END OF FILE */
