/*******************************************************************************
* File Name: Temp_Sensor_PM.c
* Version 3.30
*
* Description:
*  This file provides Low power mode APIs for I2C component.
*
* Note:
*  None
*
********************************************************************************
* Copyright 2008-2012, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "Temp_Sensor_PVT.h"

Temp_Sensor_BACKUP_STRUCT Temp_Sensor_backup =
{
    Temp_Sensor_DISABLE, /* enableState */

    #if(Temp_Sensor_FF_IMPLEMENTED)
        Temp_Sensor_DEFAULT_XCFG,  /* xcfg */
        Temp_Sensor_DEFAULT_CFG,   /* cfg  */

        #if(Temp_Sensor_MODE_SLAVE_ENABLED)
            Temp_Sensor_DEFAULT_ADDR, /* addr */
        #endif /* (Temp_Sensor_MODE_SLAVE_ENABLED) */

        #if(CY_PSOC5A)
            LO8(Temp_Sensor_DEFAULT_DIVIDE_FACTOR),  /* div */
        #else
            LO8(Temp_Sensor_DEFAULT_DIVIDE_FACTOR), /* div1 */
            HI8(Temp_Sensor_DEFAULT_DIVIDE_FACTOR), /* div2 */
        #endif /* (CY_PSOC5A) */

    #else  /* (Temp_Sensor_UDB_IMPLEMENTED) */
        Temp_Sensor_DEFAULT_CFG,    /* control */

        #if(CY_UDB_V0)
            Temp_Sensor_INT_ENABLE_MASK, /* aux_ctl */

            #if(Temp_Sensor_MODE_SLAVE_ENABLED)
                Temp_Sensor_DEFAULT_ADDR, /* addr_d0 */
            #endif /* (Temp_Sensor_MODE_SLAVE_ENABLED) */
        #endif /* (CY_UDB_V0) */
    #endif /* (Temp_Sensor_FF_IMPLEMENTED) */

    #if(Temp_Sensor_TIMEOUT_ENABLED)
        Temp_Sensor_DEFAULT_TMOUT_PERIOD,
        Temp_Sensor_DEFAULT_TMOUT_INTR_MASK,

        #if(Temp_Sensor_TIMEOUT_PRESCALER_ENABLED && CY_UDB_V0)
            Temp_Sensor_DEFAULT_TMOUT_PRESCALER_PRD,
        #endif /* (Temp_Sensor_TIMEOUT_PRESCALER_ENABLED) */

    #endif /* (Temp_Sensor_TIMEOUT_ENABLED) */
};

#if((Temp_Sensor_FF_IMPLEMENTED) && (Temp_Sensor_WAKEUP_ENABLED))
    volatile uint8 Temp_Sensor_wakeupSource;
#endif /* ((Temp_Sensor_FF_IMPLEMENTED) && (Temp_Sensor_WAKEUP_ENABLED)) */


/*******************************************************************************
* Function Name: Temp_Sensor_SaveConfig
********************************************************************************
*
* Summary:
*  Wakeup on address match enabled: disables I2C Master(if was enabled before go
*  to sleep), enables I2C backup regulator. Waits while on-going transaction be
*  will completed and I2C will be ready go to sleep. All incoming transaction
*  will be NACKed till power down will be asserted. The address match event
*  wakes up the chip.
*  Wakeup on address match disabled: saves I2C configuration and non-retention
*  register values.
*
* Parameters:
*  None
*
* Return:
*  None
*
* Global Variables:
*  Temp_Sensor_backup - used to save component configuration and
*       none-retention registers before enter sleep mode.
*
* Reentrant:
*  No
*
*******************************************************************************/
void Temp_Sensor_SaveConfig(void) 
{
    #if(Temp_Sensor_FF_IMPLEMENTED)
        #if(Temp_Sensor_WAKEUP_ENABLED)
            uint8 enableInterrupts;
        #endif /* (Temp_Sensor_WAKEUP_ENABLED) */

        /* Store regiters in either Sleep mode */
        Temp_Sensor_backup.cfg  = Temp_Sensor_CFG_REG;
        Temp_Sensor_backup.xcfg = Temp_Sensor_XCFG_REG;

        #if(Temp_Sensor_MODE_SLAVE_ENABLED)
            Temp_Sensor_backup.addr = Temp_Sensor_ADDR_REG;
        #endif /* (Temp_Sensor_MODE_SLAVE_ENABLED) */

        #if(CY_PSOC5A)
            Temp_Sensor_backup.clkDiv   = Temp_Sensor_CLKDIV_REG;
        #else
            Temp_Sensor_backup.clkDiv1  = Temp_Sensor_CLKDIV1_REG;
            Temp_Sensor_backup.clkDiv2  = Temp_Sensor_CLKDIV2_REG;
        #endif /* (CY_PSOC5A) */

        #if(Temp_Sensor_WAKEUP_ENABLED)
            /* Need to disable Master */
            Temp_Sensor_CFG_REG &= ((uint8) ~Temp_Sensor_ENABLE_MASTER);

            /* Enable the I2C regulator backup */
            enableInterrupts = CyEnterCriticalSection();
            Temp_Sensor_PWRSYS_CR1_REG |= Temp_Sensor_PWRSYS_CR1_I2C_REG_BACKUP;
            CyExitCriticalSection(enableInterrupts);

            /* 1) Set force NACK to ignore I2C transactions
               2) Wait while I2C will be ready go to Sleep
               3) These bits are cleared on wake up */
            Temp_Sensor_XCFG_REG |= Temp_Sensor_XCFG_FORCE_NACK;
            while(0u == (Temp_Sensor_XCFG_REG & Temp_Sensor_XCFG_RDY_TO_SLEEP))
            {
                ; /* Wait when block is ready to Sleep */
            }

            /* Setup wakeup interrupt */
            Temp_Sensor_DisableInt();
            (void) CyIntSetVector(Temp_Sensor_ISR_NUMBER, &Temp_Sensor_WAKEUP_ISR);
            Temp_Sensor_wakeupSource = 0u;
            Temp_Sensor_EnableInt();

        #endif /* (Temp_Sensor_WAKEUP_ENABLED) */

    #else
        /* Store only address match bit */
        Temp_Sensor_backup.control = (Temp_Sensor_CFG_REG & Temp_Sensor_CTRL_ANY_ADDRESS_MASK);

        #if(CY_UDB_V0)
            /* Store interrupt mask bits */
            Temp_Sensor_backup.intMask = Temp_Sensor_INT_MASK_REG;

            #if(Temp_Sensor_MODE & Temp_Sensor_MODE_SLAVE)
                Temp_Sensor_backup.addr = Temp_Sensor_ADDR_REG;
            #endif /* (Temp_Sensor_MODE & Temp_Sensor_MODE_SLAVE) */

        #endif /* (CY_UDB_V0) */

    #endif /* (Temp_Sensor_FF_IMPLEMENTED) */

    #if(Temp_Sensor_TIMEOUT_ENABLED)
        Temp_Sensor_TimeoutSaveConfig();   /* Save Timeout config */
    #endif /* (Temp_Sensor_TIMEOUT_ENABLED) */
}


/*******************************************************************************
* Function Name: Temp_Sensor_Sleep
********************************************************************************
*
* Summary:
*  Wakeup on address match enabled: All incoming transaction will be NACKed till
*  power down will be asserted. The address match event wakes up the chip.
*  Wakeup on address match disabled: Disables active mode power template bits or
*  clock gating as appropriate. Saves I2C configuration and non-retention
*  register values.
*  Disables I2C interrupt.
*
* Parameters:
*  None
*
* Return:
*  None
*
* Reentrant:
*  No
*
*******************************************************************************/
void Temp_Sensor_Sleep(void) 
{
    #if(Temp_Sensor_WAKEUP_ENABLED)
        /* The I2C block should be always enabled if used as wakeup source */
        Temp_Sensor_backup.enableState = Temp_Sensor_DISABLE;

        #if(Temp_Sensor_TIMEOUT_ENABLED)
            Temp_Sensor_TimeoutStop();
        #endif /* (Temp_Sensor_TIMEOUT_ENABLED) */

    #else

        Temp_Sensor_backup.enableState = ((uint8) Temp_Sensor_IS_ENABLED);

        if(Temp_Sensor_IS_ENABLED)
        {
            Temp_Sensor_Stop();
        }
    #endif /* (Temp_Sensor_WAKEUP_ENABLED) */

    Temp_Sensor_SaveConfig();
}


/*******************************************************************************
* Function Name: Temp_Sensor_RestoreConfig
********************************************************************************
*
* Summary:
*  Wakeup on address match enabled: enables I2C Master (if was enabled before go
*  to sleep), disables I2C backup regulator.
*  Wakeup on address match disabled: Restores I2C configuration and
*  non-retention register values.
*
* Parameters:
*  None
*
* Return:
*  None
*
* Global Variables:
*  Temp_Sensor_backup - used to save component configuration and
*  none-retention registers before exit sleep mode.
*
*******************************************************************************/
void Temp_Sensor_RestoreConfig(void) 
{
    #if(Temp_Sensor_FF_IMPLEMENTED)
        uint8 enableInterrupts;

        if(Temp_Sensor_CHECK_PWRSYS_I2C_BACKUP)    /* Enabled if was in Sleep */
        {
            /* Disable back-up regulator */
            enableInterrupts = CyEnterCriticalSection();
            Temp_Sensor_PWRSYS_CR1_REG &= ((uint8) ~Temp_Sensor_PWRSYS_CR1_I2C_REG_BACKUP);
            CyExitCriticalSection(enableInterrupts);

            /* Re-enable Master */
            Temp_Sensor_CFG_REG = Temp_Sensor_backup.cfg;
        }
        else /* The I2C_REG_BACKUP was cleaned by PM API: it means Hibernate or wake-up not set */
        {
            #if(Temp_Sensor_WAKEUP_ENABLED)
                /* Disable power to I2C block before register restore */
                enableInterrupts = CyEnterCriticalSection();
                Temp_Sensor_ACT_PWRMGR_REG  &= ((uint8) ~Temp_Sensor_ACT_PWR_EN);
                Temp_Sensor_STBY_PWRMGR_REG &= ((uint8) ~Temp_Sensor_STBY_PWR_EN);
                CyExitCriticalSection(enableInterrupts);

                /* Enable component after restore complete */
                Temp_Sensor_backup.enableState = Temp_Sensor_ENABLE;
            #endif /* (Temp_Sensor_WAKEUP_ENABLED) */

            /* Restore component registers: Hibernate disable power */
            Temp_Sensor_XCFG_REG = Temp_Sensor_backup.xcfg;
            Temp_Sensor_CFG_REG  = Temp_Sensor_backup.cfg;

            #if(Temp_Sensor_MODE_SLAVE_ENABLED)
                Temp_Sensor_ADDR_REG = Temp_Sensor_backup.addr;
            #endif /* (Temp_Sensor_MODE_SLAVE_ENABLED) */

            #if(CY_PSOC5A)
                Temp_Sensor_CLKDIV_REG  = Temp_Sensor_backup.clkDiv;
            #else
                Temp_Sensor_CLKDIV1_REG = Temp_Sensor_backup.clkDiv1;
                Temp_Sensor_CLKDIV2_REG = Temp_Sensor_backup.clkDiv2;
            #endif /* (CY_PSOC5A) */
        }

        #if(Temp_Sensor_WAKEUP_ENABLED)
            Temp_Sensor_DisableInt();
            (void) CyIntSetVector(Temp_Sensor_ISR_NUMBER, &Temp_Sensor_ISR);
            if(0u != Temp_Sensor_wakeupSource)
            {
                Temp_Sensor_SetPendingInt();   /* Generate interrupt to process incomming transcation */
            }
            Temp_Sensor_EnableInt();
        #endif /* (Temp_Sensor_WAKEUP_ENABLED) */

    #else

        #if(CY_UDB_V0)
            uint8 enableInterrupts;

            Temp_Sensor_INT_MASK_REG |= Temp_Sensor_backup.intMask;

            enableInterrupts = CyEnterCriticalSection();
            Temp_Sensor_INT_ENABLE_REG |= Temp_Sensor_INT_ENABLE_MASK;
            CyExitCriticalSection(enableInterrupts);

            #if(Temp_Sensor_MODE_MASTER_ENABLED)
                /* Restore Master Clock generator */
                Temp_Sensor_MCLK_PRD_REG = Temp_Sensor_DEFAULT_MCLK_PRD;
                Temp_Sensor_MCLK_CMP_REG = Temp_Sensor_DEFAULT_MCLK_CMP;
            #endif /* (Temp_Sensor_MODE_MASTER_ENABLED) */

            #if(Temp_Sensor_MODE_SLAVE_ENABLED)
                Temp_Sensor_ADDR_REG = Temp_Sensor_backup.addr;

                /* Restore slave bit counter period */
                Temp_Sensor_PERIOD_REG = Temp_Sensor_DEFAULT_PERIOD;
            #endif /* (Temp_Sensor_MODE_SLAVE_ENABLED) */

        #endif /* (CY_UDB_V0) */

        Temp_Sensor_CFG_REG = Temp_Sensor_backup.control;

    #endif /* (Temp_Sensor_FF_IMPLEMENTED) */

    #if(Temp_Sensor_TIMEOUT_ENABLED)
        Temp_Sensor_TimeoutRestoreConfig();
    #endif /* (Temp_Sensor_TIMEOUT_ENABLED) */
}


/*******************************************************************************
* Function Name: Temp_Sensor_Wakeup
********************************************************************************
*
* Summary:
*  Wakeup on address match enabled: enables I2C Master (if was enabled before go
*  to sleep) and disables I2C backup regulator.
*  Wakeup on address match disabled: Restores I2C configuration and
*  non-retention register values. Restores Active mode power template bits or
*  clock gating as appropriate.
*  The I2C interrupt remains disabled after function call.
*
* Parameters:
*  None
*
* Return:
*  None
*
* Reentrant:
*  No
*
*******************************************************************************/
void Temp_Sensor_Wakeup(void) 
{
    Temp_Sensor_RestoreConfig();   /* Restore I2C register settings */

    /* Restore component enable state */
    if(0u != Temp_Sensor_backup.enableState)
    {
        Temp_Sensor_Enable();
        Temp_Sensor_EnableInt();
    }
    else
    {
        #if(Temp_Sensor_TIMEOUT_ENABLED)
            Temp_Sensor_TimeoutEnable();
        #endif /* (Temp_Sensor_TIMEOUT_ENABLED) */
    }
}


/* [] END OF FILE */
