/*******************************************************************************
* File Name: Bluetooth.c
* Version 2.30
*
* Description:
*  This file provides all API functionality of the UART component
*
* Note:
*
********************************************************************************
* Copyright 2008-2012, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "Bluetooth.h"
#include "CyLib.h"
#if(Bluetooth_INTERNAL_CLOCK_USED)
    #include "Bluetooth_IntClock.h"
#endif /* End Bluetooth_INTERNAL_CLOCK_USED */


/***************************************
* Global data allocation
***************************************/

uint8 Bluetooth_initVar = 0u;
#if( Bluetooth_TX_ENABLED && (Bluetooth_TXBUFFERSIZE > Bluetooth_FIFO_LENGTH))
    volatile uint8 Bluetooth_txBuffer[Bluetooth_TXBUFFERSIZE];
    volatile uint8 Bluetooth_txBufferRead = 0u;
    uint8 Bluetooth_txBufferWrite = 0u;
#endif /* End Bluetooth_TX_ENABLED */
#if( ( Bluetooth_RX_ENABLED || Bluetooth_HD_ENABLED ) && \
     (Bluetooth_RXBUFFERSIZE > Bluetooth_FIFO_LENGTH) )
    volatile uint8 Bluetooth_rxBuffer[Bluetooth_RXBUFFERSIZE];
    volatile uint8 Bluetooth_rxBufferRead = 0u;
    volatile uint8 Bluetooth_rxBufferWrite = 0u;
    volatile uint8 Bluetooth_rxBufferLoopDetect = 0u;
    volatile uint8 Bluetooth_rxBufferOverflow = 0u;
    #if (Bluetooth_RXHW_ADDRESS_ENABLED)
        volatile uint8 Bluetooth_rxAddressMode = Bluetooth_RXADDRESSMODE;
        volatile uint8 Bluetooth_rxAddressDetected = 0u;
    #endif /* End EnableHWAddress */
#endif /* End Bluetooth_RX_ENABLED */


/*******************************************************************************
* Function Name: Bluetooth_Start
********************************************************************************
*
* Summary:
*  Initialize and Enable the UART component.
*  Enable the clock input to enable operation.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global variables:
*  The Bluetooth_intiVar variable is used to indicate initial
*  configuration of this component. The variable is initialized to zero (0u)
*  and set to one (1u) the first time UART_Start() is called. This allows for
*  component initialization without re-initialization in all subsequent calls
*  to the Bluetooth_Start() routine.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void Bluetooth_Start(void) 
{
    /* If not Initialized then initialize all required hardware and software */
    if(Bluetooth_initVar == 0u)
    {
        Bluetooth_Init();
        Bluetooth_initVar = 1u;
    }
    Bluetooth_Enable();
}


/*******************************************************************************
* Function Name: Bluetooth_Init
********************************************************************************
*
* Summary:
*  Initialize component's parameters to the parameters set by user in the
*  customizer of the component placed onto schematic. Usually called in
*  Bluetooth_Start().
*
* Parameters:
*  None.
*
* Return:
*  None.
*
*******************************************************************************/
void Bluetooth_Init(void) 
{
    #if(Bluetooth_RX_ENABLED || Bluetooth_HD_ENABLED)

        #if(Bluetooth_RX_INTERRUPT_ENABLED && (Bluetooth_RXBUFFERSIZE > Bluetooth_FIFO_LENGTH))
            /* Set the RX Interrupt. */
            (void)CyIntSetVector(Bluetooth_RX_VECT_NUM, &Bluetooth_RXISR);
            CyIntSetPriority(Bluetooth_RX_VECT_NUM, Bluetooth_RX_PRIOR_NUM);
        #endif /* End Bluetooth_RX_INTERRUPT_ENABLED */

        #if (Bluetooth_RXHW_ADDRESS_ENABLED)
            Bluetooth_SetRxAddressMode(Bluetooth_RXAddressMode);
            Bluetooth_SetRxAddress1(Bluetooth_RXHWADDRESS1);
            Bluetooth_SetRxAddress2(Bluetooth_RXHWADDRESS2);
        #endif /* End Bluetooth_RXHW_ADDRESS_ENABLED */

        /* Init Count7 period */
        Bluetooth_RXBITCTR_PERIOD_REG = Bluetooth_RXBITCTR_INIT;
        /* Configure the Initial RX interrupt mask */
        Bluetooth_RXSTATUS_MASK_REG  = Bluetooth_INIT_RX_INTERRUPTS_MASK;
    #endif /* End Bluetooth_RX_ENABLED || Bluetooth_HD_ENABLED*/

    #if(Bluetooth_TX_ENABLED)
        #if(Bluetooth_TX_INTERRUPT_ENABLED && (Bluetooth_TXBUFFERSIZE > Bluetooth_FIFO_LENGTH))
            /* Set the TX Interrupt. */
            (void)CyIntSetVector(Bluetooth_TX_VECT_NUM, &Bluetooth_TXISR);
            CyIntSetPriority(Bluetooth_TX_VECT_NUM, Bluetooth_TX_PRIOR_NUM);
        #endif /* End Bluetooth_TX_INTERRUPT_ENABLED */

        /* Write Counter Value for TX Bit Clk Generator*/
        #if(Bluetooth_TXCLKGEN_DP)
            Bluetooth_TXBITCLKGEN_CTR_REG = Bluetooth_BIT_CENTER;
            Bluetooth_TXBITCLKTX_COMPLETE_REG = (Bluetooth_NUMBER_OF_DATA_BITS +
                        Bluetooth_NUMBER_OF_START_BIT) * Bluetooth_OVER_SAMPLE_COUNT;
        #else
            Bluetooth_TXBITCTR_PERIOD_REG = ((Bluetooth_NUMBER_OF_DATA_BITS +
                        Bluetooth_NUMBER_OF_START_BIT) * Bluetooth_OVER_SAMPLE_8) - 1u;
        #endif /* End Bluetooth_TXCLKGEN_DP */

        /* Configure the Initial TX interrupt mask */
        #if(Bluetooth_TX_INTERRUPT_ENABLED && (Bluetooth_TXBUFFERSIZE > Bluetooth_FIFO_LENGTH))
            Bluetooth_TXSTATUS_MASK_REG = Bluetooth_TX_STS_FIFO_EMPTY;
        #else
            Bluetooth_TXSTATUS_MASK_REG = Bluetooth_INIT_TX_INTERRUPTS_MASK;
        #endif /*End Bluetooth_TX_INTERRUPT_ENABLED*/

    #endif /* End Bluetooth_TX_ENABLED */

    #if(Bluetooth_PARITY_TYPE_SW)  /* Write Parity to Control Register */
        Bluetooth_WriteControlRegister( \
            (Bluetooth_ReadControlRegister() & (uint8)~Bluetooth_CTRL_PARITY_TYPE_MASK) | \
            (uint8)(Bluetooth_PARITY_TYPE << Bluetooth_CTRL_PARITY_TYPE0_SHIFT) );
    #endif /* End Bluetooth_PARITY_TYPE_SW */
}


/*******************************************************************************
* Function Name: Bluetooth_Enable
********************************************************************************
*
* Summary:
*  Enables the UART block operation
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global Variables:
*  Bluetooth_rxAddressDetected - set to initial state (0).
*
*******************************************************************************/
void Bluetooth_Enable(void) 
{
    uint8 enableInterrupts;
    enableInterrupts = CyEnterCriticalSection();

    #if(Bluetooth_RX_ENABLED || Bluetooth_HD_ENABLED)
        /*RX Counter (Count7) Enable */
        Bluetooth_RXBITCTR_CONTROL_REG |= Bluetooth_CNTR_ENABLE;
        /* Enable the RX Interrupt. */
        Bluetooth_RXSTATUS_ACTL_REG  |= Bluetooth_INT_ENABLE;
        #if(Bluetooth_RX_INTERRUPT_ENABLED && (Bluetooth_RXBUFFERSIZE > Bluetooth_FIFO_LENGTH))
            CyIntEnable(Bluetooth_RX_VECT_NUM);
            #if (Bluetooth_RXHW_ADDRESS_ENABLED)
                Bluetooth_rxAddressDetected = 0u;
            #endif /* End Bluetooth_RXHW_ADDRESS_ENABLED */
        #endif /* End Bluetooth_RX_INTERRUPT_ENABLED */
    #endif /* End Bluetooth_RX_ENABLED || Bluetooth_HD_ENABLED*/

    #if(Bluetooth_TX_ENABLED)
        /*TX Counter (DP/Count7) Enable */
        #if(!Bluetooth_TXCLKGEN_DP)
            Bluetooth_TXBITCTR_CONTROL_REG |= Bluetooth_CNTR_ENABLE;
        #endif /* End Bluetooth_TXCLKGEN_DP */
        /* Enable the TX Interrupt. */
        Bluetooth_TXSTATUS_ACTL_REG |= Bluetooth_INT_ENABLE;
        #if(Bluetooth_TX_INTERRUPT_ENABLED && (Bluetooth_TXBUFFERSIZE > Bluetooth_FIFO_LENGTH))
            CyIntEnable(Bluetooth_TX_VECT_NUM);
        #endif /* End Bluetooth_TX_INTERRUPT_ENABLED*/
     #endif /* End Bluetooth_TX_ENABLED */

    #if(Bluetooth_INTERNAL_CLOCK_USED)
        /* Enable the clock. */
        Bluetooth_IntClock_Start();
    #endif /* End Bluetooth_INTERNAL_CLOCK_USED */

    CyExitCriticalSection(enableInterrupts);
}


/*******************************************************************************
* Function Name: Bluetooth_Stop
********************************************************************************
*
* Summary:
*  Disable the UART component
*
* Parameters:
*  None.
*
* Return:
*  None.
*
*******************************************************************************/
void Bluetooth_Stop(void) 
{
    uint8 enableInterrupts;
    enableInterrupts = CyEnterCriticalSection();

    /* Write Bit Counter Disable */
    #if(Bluetooth_RX_ENABLED || Bluetooth_HD_ENABLED)
        Bluetooth_RXBITCTR_CONTROL_REG &= (uint8)~Bluetooth_CNTR_ENABLE;
    #endif /* End Bluetooth_RX_ENABLED */

    #if(Bluetooth_TX_ENABLED)
        #if(!Bluetooth_TXCLKGEN_DP)
            Bluetooth_TXBITCTR_CONTROL_REG &= (uint8)~Bluetooth_CNTR_ENABLE;
        #endif /* End Bluetooth_TXCLKGEN_DP */
    #endif /* Bluetooth_TX_ENABLED */

    #if(Bluetooth_INTERNAL_CLOCK_USED)
        /* Disable the clock. */
        Bluetooth_IntClock_Stop();
    #endif /* End Bluetooth_INTERNAL_CLOCK_USED */

    /* Disable internal interrupt component */
    #if(Bluetooth_RX_ENABLED || Bluetooth_HD_ENABLED)
        Bluetooth_RXSTATUS_ACTL_REG  &= (uint8)~Bluetooth_INT_ENABLE;
        #if(Bluetooth_RX_INTERRUPT_ENABLED && (Bluetooth_RXBUFFERSIZE > Bluetooth_FIFO_LENGTH))
            Bluetooth_DisableRxInt();
        #endif /* End Bluetooth_RX_INTERRUPT_ENABLED */
    #endif /* End Bluetooth_RX_ENABLED */

    #if(Bluetooth_TX_ENABLED)
        Bluetooth_TXSTATUS_ACTL_REG &= (uint8)~Bluetooth_INT_ENABLE;
        #if(Bluetooth_TX_INTERRUPT_ENABLED && (Bluetooth_TXBUFFERSIZE > Bluetooth_FIFO_LENGTH))
            Bluetooth_DisableTxInt();
        #endif /* End Bluetooth_TX_INTERRUPT_ENABLED */
    #endif /* End Bluetooth_TX_ENABLED */

    CyExitCriticalSection(enableInterrupts);
}


/*******************************************************************************
* Function Name: Bluetooth_ReadControlRegister
********************************************************************************
*
* Summary:
*  Read the current state of the control register
*
* Parameters:
*  None.
*
* Return:
*  Current state of the control register.
*
*******************************************************************************/
uint8 Bluetooth_ReadControlRegister(void) 
{
    #if( Bluetooth_CONTROL_REG_REMOVED )
        return(0u);
    #else
        return(Bluetooth_CONTROL_REG);
    #endif /* End Bluetooth_CONTROL_REG_REMOVED */
}


/*******************************************************************************
* Function Name: Bluetooth_WriteControlRegister
********************************************************************************
*
* Summary:
*  Writes an 8-bit value into the control register
*
* Parameters:
*  control:  control register value
*
* Return:
*  None.
*
*******************************************************************************/
void  Bluetooth_WriteControlRegister(uint8 control) 
{
    #if( Bluetooth_CONTROL_REG_REMOVED )
        if(control != 0u) { }      /* release compiler warning */
    #else
       Bluetooth_CONTROL_REG = control;
    #endif /* End Bluetooth_CONTROL_REG_REMOVED */
}


#if(Bluetooth_RX_ENABLED || Bluetooth_HD_ENABLED)

    #if(Bluetooth_RX_INTERRUPT_ENABLED)

        /*******************************************************************************
        * Function Name: Bluetooth_EnableRxInt
        ********************************************************************************
        *
        * Summary:
        *  Enable RX interrupt generation
        *
        * Parameters:
        *  None.
        *
        * Return:
        *  None.
        *
        * Theory:
        *  Enable the interrupt output -or- the interrupt component itself
        *
        *******************************************************************************/
        void Bluetooth_EnableRxInt(void) 
        {
            CyIntEnable(Bluetooth_RX_VECT_NUM);
        }


        /*******************************************************************************
        * Function Name: Bluetooth_DisableRxInt
        ********************************************************************************
        *
        * Summary:
        *  Disable RX interrupt generation
        *
        * Parameters:
        *  None.
        *
        * Return:
        *  None.
        *
        * Theory:
        *  Disable the interrupt output -or- the interrupt component itself
        *
        *******************************************************************************/
        void Bluetooth_DisableRxInt(void) 
        {
            CyIntDisable(Bluetooth_RX_VECT_NUM);
        }

    #endif /* Bluetooth_RX_INTERRUPT_ENABLED */


    /*******************************************************************************
    * Function Name: Bluetooth_SetRxInterruptMode
    ********************************************************************************
    *
    * Summary:
    *  Configure which status bits trigger an interrupt event
    *
    * Parameters:
    *  IntSrc:  An or'd combination of the desired status bit masks (defined in
    *           the header file)
    *
    * Return:
    *  None.
    *
    * Theory:
    *  Enables the output of specific status bits to the interrupt controller
    *
    *******************************************************************************/
    void Bluetooth_SetRxInterruptMode(uint8 intSrc) 
    {
        Bluetooth_RXSTATUS_MASK_REG  = intSrc;
    }


    /*******************************************************************************
    * Function Name: Bluetooth_ReadRxData
    ********************************************************************************
    *
    * Summary:
    *  Returns data in RX Data register without checking status register to
    *  determine if data is valid
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  Received data from RX register
    *
    * Global Variables:
    *  Bluetooth_rxBuffer - RAM buffer pointer for save received data.
    *  Bluetooth_rxBufferWrite - cyclic index for write to rxBuffer,
    *     checked to identify new data.
    *  Bluetooth_rxBufferRead - cyclic index for read from rxBuffer,
    *     incremented after each byte has been read from buffer.
    *  Bluetooth_rxBufferLoopDetect - creared if loop condition was detected
    *     in RX ISR.
    *
    * Reentrant:
    *  No.
    *
    *******************************************************************************/
    uint8 Bluetooth_ReadRxData(void) 
    {
        uint8 rxData;

        #if(Bluetooth_RXBUFFERSIZE > Bluetooth_FIFO_LENGTH)
            uint8 loc_rxBufferRead;
            uint8 loc_rxBufferWrite;
            /* Protect variables that could change on interrupt. */
            /* Disable Rx interrupt. */
            #if(Bluetooth_RX_INTERRUPT_ENABLED)
                Bluetooth_DisableRxInt();
            #endif /* Bluetooth_RX_INTERRUPT_ENABLED */
            loc_rxBufferRead = Bluetooth_rxBufferRead;
            loc_rxBufferWrite = Bluetooth_rxBufferWrite;

            if( (Bluetooth_rxBufferLoopDetect != 0u) || (loc_rxBufferRead != loc_rxBufferWrite) )
            {
                rxData = Bluetooth_rxBuffer[loc_rxBufferRead];
                loc_rxBufferRead++;

                if(loc_rxBufferRead >= Bluetooth_RXBUFFERSIZE)
                {
                    loc_rxBufferRead = 0u;
                }
                /* Update the real pointer */
                Bluetooth_rxBufferRead = loc_rxBufferRead;

                if(Bluetooth_rxBufferLoopDetect != 0u )
                {
                    Bluetooth_rxBufferLoopDetect = 0u;
                    #if( (Bluetooth_RX_INTERRUPT_ENABLED) && (Bluetooth_FLOW_CONTROL != 0u) && \
                         (Bluetooth_RXBUFFERSIZE > Bluetooth_FIFO_LENGTH) )
                        /* When Hardware Flow Control selected - return RX mask */
                        #if( Bluetooth_HD_ENABLED )
                            if((Bluetooth_CONTROL_REG & Bluetooth_CTRL_HD_SEND) == 0u)
                            {   /* In Half duplex mode return RX mask only in RX
                                *  configuration set, otherwise
                                *  mask will be returned in LoadRxConfig() API.
                                */
                                Bluetooth_RXSTATUS_MASK_REG  |= Bluetooth_RX_STS_FIFO_NOTEMPTY;
                            }
                        #else
                            Bluetooth_RXSTATUS_MASK_REG  |= Bluetooth_RX_STS_FIFO_NOTEMPTY;
                        #endif /* end Bluetooth_HD_ENABLED */
                    #endif /* Bluetooth_RX_INTERRUPT_ENABLED and Hardware flow control*/
                }
            }
            else
            {   /* Needs to check status for RX_STS_FIFO_NOTEMPTY bit*/
                rxData = Bluetooth_RXDATA_REG;
            }

            /* Enable Rx interrupt. */
            #if(Bluetooth_RX_INTERRUPT_ENABLED)
                Bluetooth_EnableRxInt();
            #endif /* End Bluetooth_RX_INTERRUPT_ENABLED */

        #else /* Bluetooth_RXBUFFERSIZE > Bluetooth_FIFO_LENGTH */

            /* Needs to check status for RX_STS_FIFO_NOTEMPTY bit*/
            rxData = Bluetooth_RXDATA_REG;

        #endif /* Bluetooth_RXBUFFERSIZE > Bluetooth_FIFO_LENGTH */

        return(rxData);
    }


    /*******************************************************************************
    * Function Name: Bluetooth_ReadRxStatus
    ********************************************************************************
    *
    * Summary:
    *  Read the current state of the status register
    *  And detect software buffer overflow.
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  Current state of the status register.
    *
    * Global Variables:
    *  Bluetooth_rxBufferOverflow - used to indicate overload condition.
    *   It set to one in RX interrupt when there isn?t free space in
    *   Bluetooth_rxBufferRead to write new data. This condition returned
    *   and cleared to zero by this API as an
    *   Bluetooth_RX_STS_SOFT_BUFF_OVER bit along with RX Status register
    *   bits.
    *
    *******************************************************************************/
    uint8 Bluetooth_ReadRxStatus(void) 
    {
        uint8 status;

        status = Bluetooth_RXSTATUS_REG & Bluetooth_RX_HW_MASK;

        #if(Bluetooth_RXBUFFERSIZE > Bluetooth_FIFO_LENGTH)
            if( Bluetooth_rxBufferOverflow != 0u )
            {
                status |= Bluetooth_RX_STS_SOFT_BUFF_OVER;
                Bluetooth_rxBufferOverflow = 0u;
            }
        #endif /* Bluetooth_RXBUFFERSIZE */

        return(status);
    }


    /*******************************************************************************
    * Function Name: Bluetooth_GetChar
    ********************************************************************************
    *
    * Summary:
    *  Reads UART RX buffer immediately, if data is not available or an error
    *  condition exists, zero is returned; otherwise, character is read and
    *  returned.
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  Character read from UART RX buffer. ASCII characters from 1 to 255 are valid.
    *  A returned zero signifies an error condition or no data available.
    *
    * Global Variables:
    *  Bluetooth_rxBuffer - RAM buffer pointer for save received data.
    *  Bluetooth_rxBufferWrite - cyclic index for write to rxBuffer,
    *     checked to identify new data.
    *  Bluetooth_rxBufferRead - cyclic index for read from rxBuffer,
    *     incremented after each byte has been read from buffer.
    *  Bluetooth_rxBufferLoopDetect - creared if loop condition was detected
    *     in RX ISR.
    *
    * Reentrant:
    *  No.
    *
    *******************************************************************************/
    uint8 Bluetooth_GetChar(void) 
    {
        uint8 rxData = 0u;
        uint8 rxStatus;

        #if(Bluetooth_RXBUFFERSIZE > Bluetooth_FIFO_LENGTH)
            uint8 loc_rxBufferRead;
            uint8 loc_rxBufferWrite;
            /* Protect variables that could change on interrupt. */
            /* Disable Rx interrupt. */
            #if(Bluetooth_RX_INTERRUPT_ENABLED)
                Bluetooth_DisableRxInt();
            #endif /* Bluetooth_RX_INTERRUPT_ENABLED */
            loc_rxBufferRead = Bluetooth_rxBufferRead;
            loc_rxBufferWrite = Bluetooth_rxBufferWrite;

            if( (Bluetooth_rxBufferLoopDetect != 0u) || (loc_rxBufferRead != loc_rxBufferWrite) )
            {
                rxData = Bluetooth_rxBuffer[loc_rxBufferRead];
                loc_rxBufferRead++;
                if(loc_rxBufferRead >= Bluetooth_RXBUFFERSIZE)
                {
                    loc_rxBufferRead = 0u;
                }
                /* Update the real pointer */
                Bluetooth_rxBufferRead = loc_rxBufferRead;

                if(Bluetooth_rxBufferLoopDetect > 0u )
                {
                    Bluetooth_rxBufferLoopDetect = 0u;
                    #if( (Bluetooth_RX_INTERRUPT_ENABLED) && (Bluetooth_FLOW_CONTROL != 0u) )
                        /* When Hardware Flow Control selected - return RX mask */
                        #if( Bluetooth_HD_ENABLED )
                            if((Bluetooth_CONTROL_REG & Bluetooth_CTRL_HD_SEND) == 0u)
                            {   /* In Half duplex mode return RX mask only if
                                *  RX configuration set, otherwise
                                *  mask will be returned in LoadRxConfig() API.
                                */
                                Bluetooth_RXSTATUS_MASK_REG  |= Bluetooth_RX_STS_FIFO_NOTEMPTY;
                            }
                        #else
                            Bluetooth_RXSTATUS_MASK_REG  |= Bluetooth_RX_STS_FIFO_NOTEMPTY;
                        #endif /* end Bluetooth_HD_ENABLED */
                    #endif /* Bluetooth_RX_INTERRUPT_ENABLED and Hardware flow control*/
                }

            }
            else
            {   rxStatus = Bluetooth_RXSTATUS_REG;
                if((rxStatus & Bluetooth_RX_STS_FIFO_NOTEMPTY) != 0u)
                {   /* Read received data from FIFO*/
                    rxData = Bluetooth_RXDATA_REG;
                    /*Check status on error*/
                    if((rxStatus & (Bluetooth_RX_STS_BREAK | Bluetooth_RX_STS_PAR_ERROR |
                                   Bluetooth_RX_STS_STOP_ERROR | Bluetooth_RX_STS_OVERRUN)) != 0u)
                    {
                        rxData = 0u;
                    }
                }
            }

            /* Enable Rx interrupt. */
            #if(Bluetooth_RX_INTERRUPT_ENABLED)
                Bluetooth_EnableRxInt();
            #endif /* Bluetooth_RX_INTERRUPT_ENABLED */

        #else /* Bluetooth_RXBUFFERSIZE > Bluetooth_FIFO_LENGTH */

            rxStatus =Bluetooth_RXSTATUS_REG;
            if((rxStatus & Bluetooth_RX_STS_FIFO_NOTEMPTY) != 0u)
            {   /* Read received data from FIFO*/
                rxData = Bluetooth_RXDATA_REG;
                /*Check status on error*/
                if((rxStatus & (Bluetooth_RX_STS_BREAK | Bluetooth_RX_STS_PAR_ERROR |
                               Bluetooth_RX_STS_STOP_ERROR | Bluetooth_RX_STS_OVERRUN)) != 0u)
                {
                    rxData = 0u;
                }
            }
        #endif /* Bluetooth_RXBUFFERSIZE > Bluetooth_FIFO_LENGTH */

        return(rxData);
    }


    /*******************************************************************************
    * Function Name: Bluetooth_GetByte
    ********************************************************************************
    *
    * Summary:
    *  Grab the next available byte of data from the recieve FIFO
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  MSB contains Status Register and LSB contains UART RX data
    *
    * Reentrant:
    *  No.
    *
    *******************************************************************************/
    uint16 Bluetooth_GetByte(void) 
    {
        return ( ((uint16)Bluetooth_ReadRxStatus() << 8u) | Bluetooth_ReadRxData() );
    }


    /*******************************************************************************
    * Function Name: Bluetooth_GetRxBufferSize
    ********************************************************************************
    *
    * Summary:
    *  Determine the amount of bytes left in the RX buffer and return the count in
    *  bytes
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  uint8: Integer count of the number of bytes left
    *  in the RX buffer
    *
    * Global Variables:
    *  Bluetooth_rxBufferWrite - used to calculate left bytes.
    *  Bluetooth_rxBufferRead - used to calculate left bytes.
    *  Bluetooth_rxBufferLoopDetect - checked to decide left bytes amount.
    *
    * Reentrant:
    *  No.
    *
    * Theory:
    *  Allows the user to find out how full the RX Buffer is.
    *
    *******************************************************************************/
    uint8 Bluetooth_GetRxBufferSize(void)
                                                            
    {
        uint8 size;

        #if(Bluetooth_RXBUFFERSIZE > Bluetooth_FIFO_LENGTH)

            /* Disable Rx interrupt. */
            /* Protect variables that could change on interrupt. */
            #if(Bluetooth_RX_INTERRUPT_ENABLED)
                Bluetooth_DisableRxInt();
            #endif /* Bluetooth_RX_INTERRUPT_ENABLED */

            if(Bluetooth_rxBufferRead == Bluetooth_rxBufferWrite)
            {
                if(Bluetooth_rxBufferLoopDetect > 0u)
                {
                    size = Bluetooth_RXBUFFERSIZE;
                }
                else
                {
                    size = 0u;
                }
            }
            else if(Bluetooth_rxBufferRead < Bluetooth_rxBufferWrite)
            {
                size = (Bluetooth_rxBufferWrite - Bluetooth_rxBufferRead);
            }
            else
            {
                size = (Bluetooth_RXBUFFERSIZE - Bluetooth_rxBufferRead) + Bluetooth_rxBufferWrite;
            }

            /* Enable Rx interrupt. */
            #if(Bluetooth_RX_INTERRUPT_ENABLED)
                Bluetooth_EnableRxInt();
            #endif /* End Bluetooth_RX_INTERRUPT_ENABLED */

        #else /* Bluetooth_RXBUFFERSIZE > Bluetooth_FIFO_LENGTH */

            /* We can only know if there is data in the fifo. */
            size = ((Bluetooth_RXSTATUS_REG & Bluetooth_RX_STS_FIFO_NOTEMPTY) != 0u) ? 1u : 0u;

        #endif /* End Bluetooth_RXBUFFERSIZE > Bluetooth_FIFO_LENGTH */

        return(size);
    }


    /*******************************************************************************
    * Function Name: Bluetooth_ClearRxBuffer
    ********************************************************************************
    *
    * Summary:
    *  Clears the RX RAM buffer by setting the read and write pointers both to zero.
    *  Clears hardware RX FIFO.
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  Bluetooth_rxBufferWrite - cleared to zero.
    *  Bluetooth_rxBufferRead - cleared to zero.
    *  Bluetooth_rxBufferLoopDetect - cleared to zero.
    *  Bluetooth_rxBufferOverflow - cleared to zero.
    *
    * Reentrant:
    *  No.
    *
    * Theory:
    *  Setting the pointers to zero makes the system believe there is no data to
    *  read and writing will resume at address 0 overwriting any data that may
    *  have remained in the RAM.
    *
    * Side Effects:
    *  Any received data not read from the RAM or FIFO buffer will be lost.
    *******************************************************************************/
    void Bluetooth_ClearRxBuffer(void) 
    {
        uint8 enableInterrupts;

        /* clear the HW FIFO */
        /* Enter critical section */
        enableInterrupts = CyEnterCriticalSection();
        Bluetooth_RXDATA_AUX_CTL_REG |=  Bluetooth_RX_FIFO_CLR;
        Bluetooth_RXDATA_AUX_CTL_REG &= (uint8)~Bluetooth_RX_FIFO_CLR;
        /* Exit critical section */
        CyExitCriticalSection(enableInterrupts);

        #if(Bluetooth_RXBUFFERSIZE > Bluetooth_FIFO_LENGTH)
            /* Disable Rx interrupt. */
            /* Protect variables that could change on interrupt. */
            #if(Bluetooth_RX_INTERRUPT_ENABLED)
                Bluetooth_DisableRxInt();
            #endif /* End Bluetooth_RX_INTERRUPT_ENABLED */

            Bluetooth_rxBufferRead = 0u;
            Bluetooth_rxBufferWrite = 0u;
            Bluetooth_rxBufferLoopDetect = 0u;
            Bluetooth_rxBufferOverflow = 0u;

            /* Enable Rx interrupt. */
            #if(Bluetooth_RX_INTERRUPT_ENABLED)
                Bluetooth_EnableRxInt();
            #endif /* End Bluetooth_RX_INTERRUPT_ENABLED */
        #endif /* End Bluetooth_RXBUFFERSIZE > Bluetooth_FIFO_LENGTH */

    }


    /*******************************************************************************
    * Function Name: Bluetooth_SetRxAddressMode
    ********************************************************************************
    *
    * Summary:
    *  Set the receive addressing mode
    *
    * Parameters:
    *  addressMode: Enumerated value indicating the mode of RX addressing
    *  Bluetooth__B_UART__AM_SW_BYTE_BYTE -  Software Byte-by-Byte address
    *                                               detection
    *  Bluetooth__B_UART__AM_SW_DETECT_TO_BUFFER - Software Detect to Buffer
    *                                               address detection
    *  Bluetooth__B_UART__AM_HW_BYTE_BY_BYTE - Hardware Byte-by-Byte address
    *                                               detection
    *  Bluetooth__B_UART__AM_HW_DETECT_TO_BUFFER - Hardware Detect to Buffer
    *                                               address detection
    *  Bluetooth__B_UART__AM_NONE - No address detection
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  Bluetooth_rxAddressMode - the parameter stored in this variable for
    *   the farther usage in RX ISR.
    *  Bluetooth_rxAddressDetected - set to initial state (0).
    *
    *******************************************************************************/
    void Bluetooth_SetRxAddressMode(uint8 addressMode)
                                                        
    {
        #if(Bluetooth_RXHW_ADDRESS_ENABLED)
            #if(Bluetooth_CONTROL_REG_REMOVED)
                if(addressMode != 0u) { }     /* release compiler warning */
            #else /* Bluetooth_CONTROL_REG_REMOVED */
                uint8 tmpCtrl;
                tmpCtrl = Bluetooth_CONTROL_REG & (uint8)~Bluetooth_CTRL_RXADDR_MODE_MASK;
                tmpCtrl |= (uint8)(addressMode << Bluetooth_CTRL_RXADDR_MODE0_SHIFT);
                Bluetooth_CONTROL_REG = tmpCtrl;
                #if(Bluetooth_RX_INTERRUPT_ENABLED && \
                   (Bluetooth_RXBUFFERSIZE > Bluetooth_FIFO_LENGTH) )
                    Bluetooth_rxAddressMode = addressMode;
                    Bluetooth_rxAddressDetected = 0u;
                #endif /* End Bluetooth_RXBUFFERSIZE > Bluetooth_FIFO_LENGTH*/
            #endif /* End Bluetooth_CONTROL_REG_REMOVED */
        #else /* Bluetooth_RXHW_ADDRESS_ENABLED */
            if(addressMode != 0u) { }     /* release compiler warning */
        #endif /* End Bluetooth_RXHW_ADDRESS_ENABLED */
    }


    /*******************************************************************************
    * Function Name: Bluetooth_SetRxAddress1
    ********************************************************************************
    *
    * Summary:
    *  Set the first hardware address compare value
    *
    * Parameters:
    *  address
    *
    * Return:
    *  None.
    *
    *******************************************************************************/
    void Bluetooth_SetRxAddress1(uint8 address) 

    {
        Bluetooth_RXADDRESS1_REG = address;
    }


    /*******************************************************************************
    * Function Name: Bluetooth_SetRxAddress2
    ********************************************************************************
    *
    * Summary:
    *  Set the second hardware address compare value
    *
    * Parameters:
    *  address
    *
    * Return:
    *  None.
    *
    *******************************************************************************/
    void Bluetooth_SetRxAddress2(uint8 address) 
    {
        Bluetooth_RXADDRESS2_REG = address;
    }

#endif  /* Bluetooth_RX_ENABLED || Bluetooth_HD_ENABLED*/


#if( (Bluetooth_TX_ENABLED) || (Bluetooth_HD_ENABLED) )

    #if(Bluetooth_TX_INTERRUPT_ENABLED)

        /*******************************************************************************
        * Function Name: Bluetooth_EnableTxInt
        ********************************************************************************
        *
        * Summary:
        *  Enable TX interrupt generation
        *
        * Parameters:
        *  None.
        *
        * Return:
        *  None.
        *
        * Theory:
        *  Enable the interrupt output -or- the interrupt component itself
        *
        *******************************************************************************/
        void Bluetooth_EnableTxInt(void) 
        {
            CyIntEnable(Bluetooth_TX_VECT_NUM);
        }


        /*******************************************************************************
        * Function Name: Bluetooth_DisableTxInt
        ********************************************************************************
        *
        * Summary:
        *  Disable TX interrupt generation
        *
        * Parameters:
        *  None.
        *
        * Return:
        *  None.
        *
        * Theory:
        *  Disable the interrupt output -or- the interrupt component itself
        *
        *******************************************************************************/
        void Bluetooth_DisableTxInt(void) 
        {
            CyIntDisable(Bluetooth_TX_VECT_NUM);
        }

    #endif /* Bluetooth_TX_INTERRUPT_ENABLED */


    /*******************************************************************************
    * Function Name: Bluetooth_SetTxInterruptMode
    ********************************************************************************
    *
    * Summary:
    *  Configure which status bits trigger an interrupt event
    *
    * Parameters:
    *  intSrc: An or'd combination of the desired status bit masks (defined in
    *          the header file)
    *
    * Return:
    *  None.
    *
    * Theory:
    *  Enables the output of specific status bits to the interrupt controller
    *
    *******************************************************************************/
    void Bluetooth_SetTxInterruptMode(uint8 intSrc) 
    {
        Bluetooth_TXSTATUS_MASK_REG = intSrc;
    }


    /*******************************************************************************
    * Function Name: Bluetooth_WriteTxData
    ********************************************************************************
    *
    * Summary:
    *  Write a byte of data to the Transmit FIFO or TX buffer to be sent when the
    *  bus is available. WriteTxData sends a byte without checking for buffer room
    *  or status. It is up to the user to separately check status.
    *
    * Parameters:
    *  TXDataByte: byte of data to place in the transmit FIFO
    *
    * Return:
    * void
    *
    * Global Variables:
    *  Bluetooth_txBuffer - RAM buffer pointer for save data for transmission
    *  Bluetooth_txBufferWrite - cyclic index for write to txBuffer,
    *    incremented after each byte saved to buffer.
    *  Bluetooth_txBufferRead - cyclic index for read from txBuffer,
    *    checked to identify the condition to write to FIFO directly or to TX buffer
    *  Bluetooth_initVar - checked to identify that the component has been
    *    initialized.
    *
    * Reentrant:
    *  No.
    *
    *******************************************************************************/
    void Bluetooth_WriteTxData(uint8 txDataByte) 
    {
        /* If not Initialized then skip this function*/
        if(Bluetooth_initVar != 0u)
        {
            #if(Bluetooth_TXBUFFERSIZE > Bluetooth_FIFO_LENGTH)

                /* Disable Tx interrupt. */
                /* Protect variables that could change on interrupt. */
                #if(Bluetooth_TX_INTERRUPT_ENABLED)
                    Bluetooth_DisableTxInt();
                #endif /* End Bluetooth_TX_INTERRUPT_ENABLED */

                if( (Bluetooth_txBufferRead == Bluetooth_txBufferWrite) &&
                    ((Bluetooth_TXSTATUS_REG & Bluetooth_TX_STS_FIFO_FULL) == 0u) )
                {
                    /* Add directly to the FIFO. */
                    Bluetooth_TXDATA_REG = txDataByte;
                }
                else
                {
                    if(Bluetooth_txBufferWrite >= Bluetooth_TXBUFFERSIZE)
                    {
                        Bluetooth_txBufferWrite = 0u;
                    }

                    Bluetooth_txBuffer[Bluetooth_txBufferWrite] = txDataByte;

                    /* Add to the software buffer. */
                    Bluetooth_txBufferWrite++;

                }

                /* Enable Tx interrupt. */
                #if(Bluetooth_TX_INTERRUPT_ENABLED)
                    Bluetooth_EnableTxInt();
                #endif /* End Bluetooth_TX_INTERRUPT_ENABLED */

            #else /* Bluetooth_TXBUFFERSIZE > Bluetooth_FIFO_LENGTH */

                /* Add directly to the FIFO. */
                Bluetooth_TXDATA_REG = txDataByte;

            #endif /* End Bluetooth_TXBUFFERSIZE > Bluetooth_FIFO_LENGTH */
        }
    }


    /*******************************************************************************
    * Function Name: Bluetooth_ReadTxStatus
    ********************************************************************************
    *
    * Summary:
    *  Read the status register for the component
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  Contents of the status register
    *
    * Theory:
    *  This function reads the status register which is clear on read. It is up to
    *  the user to handle all bits in this return value accordingly, even if the bit
    *  was not enabled as an interrupt source the event happened and must be handled
    *  accordingly.
    *
    *******************************************************************************/
    uint8 Bluetooth_ReadTxStatus(void) 
    {
        return(Bluetooth_TXSTATUS_REG);
    }


    /*******************************************************************************
    * Function Name: Bluetooth_PutChar
    ********************************************************************************
    *
    * Summary:
    *  Wait to send byte until TX register or buffer has room.
    *
    * Parameters:
    *  txDataByte: The 8-bit data value to send across the UART.
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  Bluetooth_txBuffer - RAM buffer pointer for save data for transmission
    *  Bluetooth_txBufferWrite - cyclic index for write to txBuffer,
    *     checked to identify free space in txBuffer and incremented after each byte
    *     saved to buffer.
    *  Bluetooth_txBufferRead - cyclic index for read from txBuffer,
    *     checked to identify free space in txBuffer.
    *  Bluetooth_initVar - checked to identify that the component has been
    *     initialized.
    *
    * Reentrant:
    *  No.
    *
    * Theory:
    *  Allows the user to transmit any byte of data in a single transfer
    *
    *******************************************************************************/
    void Bluetooth_PutChar(uint8 txDataByte) 
    {
            #if(Bluetooth_TXBUFFERSIZE > Bluetooth_FIFO_LENGTH)
                /* The temporary output pointer is used since it takes two instructions
                *  to increment with a wrap, and we can't risk doing that with the real
                *  pointer and getting an interrupt in between instructions.
                */
                uint8 loc_txBufferWrite;
                uint8 loc_txBufferRead;

                do{
                    /* Block if software buffer is full, so we don't overwrite. */
                    #if ((Bluetooth_TXBUFFERSIZE > Bluetooth_MAX_BYTE_VALUE) && (CY_PSOC3))
                        /* Disable TX interrupt to protect variables that could change on interrupt */
                        CyIntDisable(Bluetooth_TX_VECT_NUM);
                    #endif /* End TXBUFFERSIZE > 255 */
                    loc_txBufferWrite = Bluetooth_txBufferWrite;
                    loc_txBufferRead = Bluetooth_txBufferRead;
                    #if ((Bluetooth_TXBUFFERSIZE > Bluetooth_MAX_BYTE_VALUE) && (CY_PSOC3))
                        /* Enable interrupt to continue transmission */
                        CyIntEnable(Bluetooth_TX_VECT_NUM);
                    #endif /* End TXBUFFERSIZE > 255 */
                }while( (loc_txBufferWrite < loc_txBufferRead) ? (loc_txBufferWrite == (loc_txBufferRead - 1u)) :
                                        ((loc_txBufferWrite - loc_txBufferRead) ==
                                        (uint8)(Bluetooth_TXBUFFERSIZE - 1u)) );

                if( (loc_txBufferRead == loc_txBufferWrite) &&
                    ((Bluetooth_TXSTATUS_REG & Bluetooth_TX_STS_FIFO_FULL) == 0u) )
                {
                    /* Add directly to the FIFO. */
                    Bluetooth_TXDATA_REG = txDataByte;
                }
                else
                {
                    if(loc_txBufferWrite >= Bluetooth_TXBUFFERSIZE)
                    {
                        loc_txBufferWrite = 0u;
                    }
                    /* Add to the software buffer. */
                    Bluetooth_txBuffer[loc_txBufferWrite] = txDataByte;
                    loc_txBufferWrite++;

                    /* Finally, update the real output pointer */
                    #if ((Bluetooth_TXBUFFERSIZE > Bluetooth_MAX_BYTE_VALUE) && (CY_PSOC3))
                        CyIntDisable(Bluetooth_TX_VECT_NUM);
                    #endif /* End TXBUFFERSIZE > 255 */
                    Bluetooth_txBufferWrite = loc_txBufferWrite;
                    #if ((Bluetooth_TXBUFFERSIZE > Bluetooth_MAX_BYTE_VALUE) && (CY_PSOC3))
                        CyIntEnable(Bluetooth_TX_VECT_NUM);
                    #endif /* End TXBUFFERSIZE > 255 */
                }

            #else /* Bluetooth_TXBUFFERSIZE > Bluetooth_FIFO_LENGTH */

                while((Bluetooth_TXSTATUS_REG & Bluetooth_TX_STS_FIFO_FULL) != 0u)
                {
                    ; /* Wait for room in the FIFO. */
                }

                /* Add directly to the FIFO. */
                Bluetooth_TXDATA_REG = txDataByte;

            #endif /* End Bluetooth_TXBUFFERSIZE > Bluetooth_FIFO_LENGTH */
    }


    /*******************************************************************************
    * Function Name: Bluetooth_PutString
    ********************************************************************************
    *
    * Summary:
    *  Write a Sequence of bytes on the Transmit line. Data comes from RAM or ROM.
    *
    * Parameters:
    *  string: char pointer to character string of Data to Send.
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  Bluetooth_initVar - checked to identify that the component has been
    *     initialized.
    *
    * Reentrant:
    *  No.
    *
    * Theory:
    *  This function will block if there is not enough memory to place the whole
    *  string, it will block until the entire string has been written to the
    *  transmit buffer.
    *
    *******************************************************************************/
    void Bluetooth_PutString(const char8 string[]) 
    {
        uint16 buf_index = 0u;
        /* If not Initialized then skip this function*/
        if(Bluetooth_initVar != 0u)
        {
            /* This is a blocking function, it will not exit until all data is sent*/
            while(string[buf_index] != (char8)0)
            {
                Bluetooth_PutChar((uint8)string[buf_index]);
                buf_index++;
            }
        }
    }


    /*******************************************************************************
    * Function Name: Bluetooth_PutArray
    ********************************************************************************
    *
    * Summary:
    *  Write a Sequence of bytes on the Transmit line. Data comes from RAM or ROM.
    *
    * Parameters:
    *  string: Address of the memory array residing in RAM or ROM.
    *  byteCount: Number of Bytes to be transmitted.
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  Bluetooth_initVar - checked to identify that the component has been
    *     initialized.
    *
    * Reentrant:
    *  No.
    *
    *******************************************************************************/
    void Bluetooth_PutArray(const uint8 string[], uint8 byteCount)
                                                                    
    {
        uint8 buf_index = 0u;
        /* If not Initialized then skip this function*/
        if(Bluetooth_initVar != 0u)
        {
            do
            {
                Bluetooth_PutChar(string[buf_index]);
                buf_index++;
            }while(buf_index < byteCount);
        }
    }


    /*******************************************************************************
    * Function Name: Bluetooth_PutCRLF
    ********************************************************************************
    *
    * Summary:
    *  Write a character and then carriage return and line feed.
    *
    * Parameters:
    *  txDataByte: uint8 Character to send.
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  Bluetooth_initVar - checked to identify that the component has been
    *     initialized.
    *
    * Reentrant:
    *  No.
    *
    *******************************************************************************/
    void Bluetooth_PutCRLF(uint8 txDataByte) 
    {
        /* If not Initialized then skip this function*/
        if(Bluetooth_initVar != 0u)
        {
            Bluetooth_PutChar(txDataByte);
            Bluetooth_PutChar(0x0Du);
            Bluetooth_PutChar(0x0Au);
        }
    }


    /*******************************************************************************
    * Function Name: Bluetooth_GetTxBufferSize
    ********************************************************************************
    *
    * Summary:
    *  Determine the amount of space left in the TX buffer and return the count in
    *  bytes
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  Integer count of the number of bytes left in the TX buffer
    *
    * Global Variables:
    *  Bluetooth_txBufferWrite - used to calculate left space.
    *  Bluetooth_txBufferRead - used to calculate left space.
    *
    * Reentrant:
    *  No.
    *
    * Theory:
    *  Allows the user to find out how full the TX Buffer is.
    *
    *******************************************************************************/
    uint8 Bluetooth_GetTxBufferSize(void)
                                                            
    {
        uint8 size;

        #if(Bluetooth_TXBUFFERSIZE > Bluetooth_FIFO_LENGTH)

            /* Disable Tx interrupt. */
            /* Protect variables that could change on interrupt. */
            #if(Bluetooth_TX_INTERRUPT_ENABLED)
                Bluetooth_DisableTxInt();
            #endif /* End Bluetooth_TX_INTERRUPT_ENABLED */

            if(Bluetooth_txBufferRead == Bluetooth_txBufferWrite)
            {
                size = 0u;
            }
            else if(Bluetooth_txBufferRead < Bluetooth_txBufferWrite)
            {
                size = (Bluetooth_txBufferWrite - Bluetooth_txBufferRead);
            }
            else
            {
                size = (Bluetooth_TXBUFFERSIZE - Bluetooth_txBufferRead) + Bluetooth_txBufferWrite;
            }

            /* Enable Tx interrupt. */
            #if(Bluetooth_TX_INTERRUPT_ENABLED)
                Bluetooth_EnableTxInt();
            #endif /* End Bluetooth_TX_INTERRUPT_ENABLED */

        #else /* Bluetooth_TXBUFFERSIZE > Bluetooth_FIFO_LENGTH */

            size = Bluetooth_TXSTATUS_REG;

            /* Is the fifo is full. */
            if((size & Bluetooth_TX_STS_FIFO_FULL) != 0u)
            {
                size = Bluetooth_FIFO_LENGTH;
            }
            else if((size & Bluetooth_TX_STS_FIFO_EMPTY) != 0u)
            {
                size = 0u;
            }
            else
            {
                /* We only know there is data in the fifo. */
                size = 1u;
            }

        #endif /* End Bluetooth_TXBUFFERSIZE > Bluetooth_FIFO_LENGTH */

        return(size);
    }


    /*******************************************************************************
    * Function Name: Bluetooth_ClearTxBuffer
    ********************************************************************************
    *
    * Summary:
    *  Clears the TX RAM buffer by setting the read and write pointers both to zero.
    *  Clears the hardware TX FIFO.  Any data present in the FIFO will not be sent.
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  Bluetooth_txBufferWrite - cleared to zero.
    *  Bluetooth_txBufferRead - cleared to zero.
    *
    * Reentrant:
    *  No.
    *
    * Theory:
    *  Setting the pointers to zero makes the system believe there is no data to
    *  read and writing will resume at address 0 overwriting any data that may have
    *  remained in the RAM.
    *
    * Side Effects:
    *  Any received data not read from the RAM buffer will be lost when overwritten.
    *
    *******************************************************************************/
    void Bluetooth_ClearTxBuffer(void) 
    {
        uint8 enableInterrupts;

        /* Enter critical section */
        enableInterrupts = CyEnterCriticalSection();
        /* clear the HW FIFO */
        Bluetooth_TXDATA_AUX_CTL_REG |=  Bluetooth_TX_FIFO_CLR;
        Bluetooth_TXDATA_AUX_CTL_REG &= (uint8)~Bluetooth_TX_FIFO_CLR;
        /* Exit critical section */
        CyExitCriticalSection(enableInterrupts);

        #if(Bluetooth_TXBUFFERSIZE > Bluetooth_FIFO_LENGTH)

            /* Disable Tx interrupt. */
            /* Protect variables that could change on interrupt. */
            #if(Bluetooth_TX_INTERRUPT_ENABLED)
                Bluetooth_DisableTxInt();
            #endif /* End Bluetooth_TX_INTERRUPT_ENABLED */

            Bluetooth_txBufferRead = 0u;
            Bluetooth_txBufferWrite = 0u;

            /* Enable Tx interrupt. */
            #if(Bluetooth_TX_INTERRUPT_ENABLED)
                Bluetooth_EnableTxInt();
            #endif /* End Bluetooth_TX_INTERRUPT_ENABLED */

        #endif /* End Bluetooth_TXBUFFERSIZE > Bluetooth_FIFO_LENGTH */
    }


    /*******************************************************************************
    * Function Name: Bluetooth_SendBreak
    ********************************************************************************
    *
    * Summary:
    *  Write a Break command to the UART
    *
    * Parameters:
    *  uint8 retMode:  Wait mode,
    *   0 - Initialize registers for Break, sends the Break signal and return
    *       imediately.
    *   1 - Wait until Break sending is complete, reinitialize registers to normal
    *       transmission mode then return.
    *   2 - Reinitialize registers to normal transmission mode then return.
    *   3 - both steps: 0 and 1
    *       init registers for Break, send Break signal
    *       wait until Break sending is complete, reinit registers to normal
    *       transmission mode then return.
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  Bluetooth_initVar - checked to identify that the component has been
    *     initialized.
    *  tx_period - static variable, used for keeping TX period configuration.
    *
    * Reentrant:
    *  No.
    *
    * Theory:
    *  SendBreak function initializes registers to send 13-bit break signal. It is
    *  important to return the registers configuration to normal for continue 8-bit
    *  operation.
    *  Trere are 3 variants for this API usage:
    *  1) SendBreak(3) - function will send the Break signal and take care on the
    *     configuration returning. Funcition will block CPU untill transmition
    *     complete.
    *  2) User may want to use bloking time if UART configured to the low speed
    *     operation
    *     Emample for this case:
    *     SendBreak(0);     - init Break signal transmition
    *         Add your code here to use CPU time
    *     SendBreak(1);     - complete Break operation
    *  3) Same to 2) but user may want to init and use the interrupt for complete
    *     break operation.
    *     Example for this case:
    *     Init TX interrupt whith "TX - On TX Complete" parameter
    *     SendBreak(0);     - init Break signal transmition
    *         Add your code here to use CPU time
    *     When interrupt appear with UART_TX_STS_COMPLETE status:
    *     SendBreak(2);     - complete Break operation
    *
    * Side Effects:
    *   Uses static variable to keep registers configuration.
    *
    *******************************************************************************/
    void Bluetooth_SendBreak(uint8 retMode) 
    {

        /* If not Initialized then skip this function*/
        if(Bluetooth_initVar != 0u)
        {
            /*Set the Counter to 13-bits and transmit a 00 byte*/
            /*When that is done then reset the counter value back*/
            uint8 tmpStat;

            #if(Bluetooth_HD_ENABLED) /* Half Duplex mode*/

                if( (retMode == Bluetooth_SEND_BREAK) ||
                    (retMode == Bluetooth_SEND_WAIT_REINIT ) )
                {
                    /* CTRL_HD_SEND_BREAK - sends break bits in HD mode*/
                    Bluetooth_WriteControlRegister(Bluetooth_ReadControlRegister() |
                                                          Bluetooth_CTRL_HD_SEND_BREAK);
                    /* Send zeros*/
                    Bluetooth_TXDATA_REG = 0u;

                    do /*wait until transmit starts*/
                    {
                        tmpStat = Bluetooth_TXSTATUS_REG;
                    }while((tmpStat & Bluetooth_TX_STS_FIFO_EMPTY) != 0u);
                }

                if( (retMode == Bluetooth_WAIT_FOR_COMPLETE_REINIT) ||
                    (retMode == Bluetooth_SEND_WAIT_REINIT) )
                {
                    do /*wait until transmit complete*/
                    {
                        tmpStat = Bluetooth_TXSTATUS_REG;
                    }while(((uint8)~tmpStat & Bluetooth_TX_STS_COMPLETE) != 0u);
                }

                if( (retMode == Bluetooth_WAIT_FOR_COMPLETE_REINIT) ||
                    (retMode == Bluetooth_REINIT) ||
                    (retMode == Bluetooth_SEND_WAIT_REINIT) )
                {
                    Bluetooth_WriteControlRegister(Bluetooth_ReadControlRegister() &
                                                  (uint8)~Bluetooth_CTRL_HD_SEND_BREAK);
                }

            #else /* Bluetooth_HD_ENABLED Full Duplex mode */

                static uint8 tx_period;

                if( (retMode == Bluetooth_SEND_BREAK) ||
                    (retMode == Bluetooth_SEND_WAIT_REINIT) )
                {
                    /* CTRL_HD_SEND_BREAK - skip to send parity bit at Break signal in Full Duplex mode*/
                    #if( (Bluetooth_PARITY_TYPE != Bluetooth__B_UART__NONE_REVB) || \
                                        (Bluetooth_PARITY_TYPE_SW != 0u) )
                        Bluetooth_WriteControlRegister(Bluetooth_ReadControlRegister() |
                                                              Bluetooth_CTRL_HD_SEND_BREAK);
                    #endif /* End Bluetooth_PARITY_TYPE != Bluetooth__B_UART__NONE_REVB  */

                    #if(Bluetooth_TXCLKGEN_DP)
                        tx_period = Bluetooth_TXBITCLKTX_COMPLETE_REG;
                        Bluetooth_TXBITCLKTX_COMPLETE_REG = Bluetooth_TXBITCTR_BREAKBITS;
                    #else
                        tx_period = Bluetooth_TXBITCTR_PERIOD_REG;
                        Bluetooth_TXBITCTR_PERIOD_REG = Bluetooth_TXBITCTR_BREAKBITS8X;
                    #endif /* End Bluetooth_TXCLKGEN_DP */

                    /* Send zeros*/
                    Bluetooth_TXDATA_REG = 0u;

                    do /* wait until transmit starts */
                    {
                        tmpStat = Bluetooth_TXSTATUS_REG;
                    }while((tmpStat & Bluetooth_TX_STS_FIFO_EMPTY) != 0u);
                }

                if( (retMode == Bluetooth_WAIT_FOR_COMPLETE_REINIT) ||
                    (retMode == Bluetooth_SEND_WAIT_REINIT) )
                {
                    do /*wait until transmit complete*/
                    {
                        tmpStat = Bluetooth_TXSTATUS_REG;
                    }while(((uint8)~tmpStat & Bluetooth_TX_STS_COMPLETE) != 0u);
                }

                if( (retMode == Bluetooth_WAIT_FOR_COMPLETE_REINIT) ||
                    (retMode == Bluetooth_REINIT) ||
                    (retMode == Bluetooth_SEND_WAIT_REINIT) )
                {

                    #if(Bluetooth_TXCLKGEN_DP)
                        Bluetooth_TXBITCLKTX_COMPLETE_REG = tx_period;
                    #else
                        Bluetooth_TXBITCTR_PERIOD_REG = tx_period;
                    #endif /* End Bluetooth_TXCLKGEN_DP */

                    #if( (Bluetooth_PARITY_TYPE != Bluetooth__B_UART__NONE_REVB) || \
                         (Bluetooth_PARITY_TYPE_SW != 0u) )
                        Bluetooth_WriteControlRegister(Bluetooth_ReadControlRegister() &
                                                      (uint8)~Bluetooth_CTRL_HD_SEND_BREAK);
                    #endif /* End Bluetooth_PARITY_TYPE != NONE */
                }
            #endif    /* End Bluetooth_HD_ENABLED */
        }
    }


    /*******************************************************************************
    * Function Name: Bluetooth_SetTxAddressMode
    ********************************************************************************
    *
    * Summary:
    *  Set the transmit addressing mode
    *
    * Parameters:
    *  addressMode: 0 -> Space
    *               1 -> Mark
    *
    * Return:
    *  None.
    *
    *******************************************************************************/
    void Bluetooth_SetTxAddressMode(uint8 addressMode) 
    {
        /* Mark/Space sending enable*/
        if(addressMode != 0u)
        {
            #if( Bluetooth_CONTROL_REG_REMOVED == 0u )
                Bluetooth_WriteControlRegister(Bluetooth_ReadControlRegister() |
                                                      Bluetooth_CTRL_MARK);
            #endif /* End Bluetooth_CONTROL_REG_REMOVED == 0u */
        }
        else
        {
            #if( Bluetooth_CONTROL_REG_REMOVED == 0u )
                Bluetooth_WriteControlRegister(Bluetooth_ReadControlRegister() &
                                                    (uint8)~Bluetooth_CTRL_MARK);
            #endif /* End Bluetooth_CONTROL_REG_REMOVED == 0u */
        }
    }

#endif  /* EndBluetooth_TX_ENABLED */

#if(Bluetooth_HD_ENABLED)


    /*******************************************************************************
    * Function Name: Bluetooth_LoadTxConfig
    ********************************************************************************
    *
    * Summary:
    *  Unloads the Rx configuration if required and loads the
    *  Tx configuration. It is the users responsibility to ensure that any
    *  transaction is complete and it is safe to unload the Tx
    *  configuration.
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  None.
    *
    * Theory:
    *  Valid only for half duplex UART.
    *
    * Side Effects:
    *  Disable RX interrupt mask, when software buffer has been used.
    *
    *******************************************************************************/
    void Bluetooth_LoadTxConfig(void) 
    {
        #if((Bluetooth_RX_INTERRUPT_ENABLED) && (Bluetooth_RXBUFFERSIZE > Bluetooth_FIFO_LENGTH))
            /* Disable RX interrupts before set TX configuration */
            Bluetooth_SetRxInterruptMode(0u);
        #endif /* Bluetooth_RX_INTERRUPT_ENABLED */

        Bluetooth_WriteControlRegister(Bluetooth_ReadControlRegister() | Bluetooth_CTRL_HD_SEND);
        Bluetooth_RXBITCTR_PERIOD_REG = Bluetooth_HD_TXBITCTR_INIT;
        #if(CY_UDB_V0) /* Manually clear status register when mode has been changed */
            /* Clear status register */
            CY_GET_REG8(Bluetooth_RXSTATUS_PTR);
        #endif /* CY_UDB_V0 */
    }


    /*******************************************************************************
    * Function Name: Bluetooth_LoadRxConfig
    ********************************************************************************
    *
    * Summary:
    *  Unloads the Tx configuration if required and loads the
    *  Rx configuration. It is the users responsibility to ensure that any
    *  transaction is complete and it is safe to unload the Rx
    *  configuration.
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  None.
    *
    * Theory:
    *  Valid only for half duplex UART
    *
    * Side Effects:
    *  Set RX interrupt mask based on customizer settings, when software buffer
    *  has been used.
    *
    *******************************************************************************/
    void Bluetooth_LoadRxConfig(void) 
    {
        Bluetooth_WriteControlRegister(Bluetooth_ReadControlRegister() &
                                                (uint8)~Bluetooth_CTRL_HD_SEND);
        Bluetooth_RXBITCTR_PERIOD_REG = Bluetooth_HD_RXBITCTR_INIT;
        #if(CY_UDB_V0) /* Manually clear status register when mode has been changed */
            /* Clear status register */
            CY_GET_REG8(Bluetooth_RXSTATUS_PTR);
        #endif /* CY_UDB_V0 */

        #if((Bluetooth_RX_INTERRUPT_ENABLED) && (Bluetooth_RXBUFFERSIZE > Bluetooth_FIFO_LENGTH))
            /* Enable RX interrupt after set RX configuration */
            Bluetooth_SetRxInterruptMode(Bluetooth_INIT_RX_INTERRUPTS_MASK);
        #endif /* Bluetooth_RX_INTERRUPT_ENABLED */
    }

#endif  /* Bluetooth_HD_ENABLED */


/* [] END OF FILE */
