/*******************************************************************************
* File Name: NC_COUNTER_PM.c  
* Version 2.30
*
*  Description:
*    This file provides the power management source code to API for the
*    Counter.  
*
*   Note:
*     None
*
*******************************************************************************
* Copyright 2008-2012, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
********************************************************************************/

#include "NC_COUNTER.h"

static NC_COUNTER_backupStruct NC_COUNTER_backup;


/*******************************************************************************
* Function Name: NC_COUNTER_SaveConfig
********************************************************************************
* Summary:
*     Save the current user configuration
*
* Parameters:  
*  void
*
* Return: 
*  void
*
* Global variables:
*  NC_COUNTER_backup:  Variables of this global structure are modified to 
*  store the values of non retention configuration registers when Sleep() API is 
*  called.
*
*******************************************************************************/
void NC_COUNTER_SaveConfig(void) 
{
    #if (!NC_COUNTER_UsingFixedFunction)
        /* Backup the UDB non-rentention registers for PSoC5A */
        #if (CY_PSOC5A)
            NC_COUNTER_backup.CounterUdb = NC_COUNTER_ReadCounter();
            NC_COUNTER_backup.CounterPeriod = NC_COUNTER_ReadPeriod();
            NC_COUNTER_backup.CompareValue = NC_COUNTER_ReadCompare();
            NC_COUNTER_backup.InterruptMaskValue = NC_COUNTER_STATUS_MASK;
        #endif /* (CY_PSOC5A) */
        
        #if (CY_PSOC3 || CY_PSOC5LP)
            NC_COUNTER_backup.CounterUdb = NC_COUNTER_ReadCounter();
            NC_COUNTER_backup.InterruptMaskValue = NC_COUNTER_STATUS_MASK;
        #endif /* (CY_PSOC3 || CY_PSOC5LP) */
        
        #if(!NC_COUNTER_ControlRegRemoved)
            NC_COUNTER_backup.CounterControlRegister = NC_COUNTER_ReadControlRegister();
        #endif /* (!NC_COUNTER_ControlRegRemoved) */
    #endif /* (!NC_COUNTER_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: NC_COUNTER_RestoreConfig
********************************************************************************
*
* Summary:
*  Restores the current user configuration.
*
* Parameters:  
*  void
*
* Return: 
*  void
*
* Global variables:
*  NC_COUNTER_backup:  Variables of this global structure are used to 
*  restore the values of non retention registers on wakeup from sleep mode.
*
*******************************************************************************/
void NC_COUNTER_RestoreConfig(void) 
{      
    #if (!NC_COUNTER_UsingFixedFunction)     
        /* Restore the UDB non-rentention registers for PSoC5A */
        #if (CY_PSOC5A)
            /* Interrupt State Backup for Critical Region*/
            uint8 NC_COUNTER_interruptState;
        
            NC_COUNTER_WriteCounter(NC_COUNTER_backup.CounterUdb);
            NC_COUNTER_WritePeriod(NC_COUNTER_backup.CounterPeriod);
            NC_COUNTER_WriteCompare(NC_COUNTER_backup.CompareValue);
            /* Enter Critical Region*/
            NC_COUNTER_interruptState = CyEnterCriticalSection();
        
            NC_COUNTER_STATUS_AUX_CTRL |= NC_COUNTER_STATUS_ACTL_INT_EN_MASK;
            /* Exit Critical Region*/
            CyExitCriticalSection(NC_COUNTER_interruptState);
            NC_COUNTER_STATUS_MASK = NC_COUNTER_backup.InterruptMaskValue;
        #endif  /* (CY_PSOC5A) */
        
        #if (CY_PSOC3 || CY_PSOC5LP)
            NC_COUNTER_WriteCounter(NC_COUNTER_backup.CounterUdb);
            NC_COUNTER_STATUS_MASK = NC_COUNTER_backup.InterruptMaskValue;
        #endif /* (CY_PSOC3 || CY_PSOC5LP) */
        
        #if(!NC_COUNTER_ControlRegRemoved)
            NC_COUNTER_WriteControlRegister(NC_COUNTER_backup.CounterControlRegister);
        #endif /* (!NC_COUNTER_ControlRegRemoved) */
    #endif /* (!NC_COUNTER_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: NC_COUNTER_Sleep
********************************************************************************
* Summary:
*     Stop and Save the user configuration
*
* Parameters:  
*  void
*
* Return: 
*  void
*
* Global variables:
*  NC_COUNTER_backup.enableState:  Is modified depending on the enable 
*  state of the block before entering sleep mode.
*
*******************************************************************************/
void NC_COUNTER_Sleep(void) 
{
    #if(!NC_COUNTER_ControlRegRemoved)
        /* Save Counter's enable state */
        if(NC_COUNTER_CTRL_ENABLE == (NC_COUNTER_CONTROL & NC_COUNTER_CTRL_ENABLE))
        {
            /* Counter is enabled */
            NC_COUNTER_backup.CounterEnableState = 1u;
        }
        else
        {
            /* Counter is disabled */
            NC_COUNTER_backup.CounterEnableState = 0u;
        }
    #else
        NC_COUNTER_backup.CounterEnableState = 1u;
        if(NC_COUNTER_backup.CounterEnableState != 0u)
        {
            NC_COUNTER_backup.CounterEnableState = 0u;
        }
    #endif /* (!NC_COUNTER_ControlRegRemoved) */
    
    NC_COUNTER_Stop();
    NC_COUNTER_SaveConfig();
}


/*******************************************************************************
* Function Name: NC_COUNTER_Wakeup
********************************************************************************
*
* Summary:
*  Restores and enables the user configuration
*  
* Parameters:  
*  void
*
* Return: 
*  void
*
* Global variables:
*  NC_COUNTER_backup.enableState:  Is used to restore the enable state of 
*  block on wakeup from sleep mode.
*
*******************************************************************************/
void NC_COUNTER_Wakeup(void) 
{
    NC_COUNTER_RestoreConfig();
    #if(!NC_COUNTER_ControlRegRemoved)
        if(NC_COUNTER_backup.CounterEnableState == 1u)
        {
            /* Enable Counter's operation */
            NC_COUNTER_Enable();
        } /* Do nothing if Counter was disabled before */    
    #endif /* (!NC_COUNTER_ControlRegRemoved) */
    
}


/* [] END OF FILE */
